// Licensed under the Apache License, Version 2.0
// <LICENSE-APACHE or http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your option.
// All files in the project carrying such notice may not be copied, modified, or distributed
// except according to those terms.
use shared::wtypes::PROPERTYKEY;
//52
DEFINE_PROPERTYKEY!{PKEY_Device_DeviceDesc,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 2}
DEFINE_PROPERTYKEY!{PKEY_Device_HardwareIds,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 3}
DEFINE_PROPERTYKEY!{PKEY_Device_CompatibleIds,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 4}
DEFINE_PROPERTYKEY!{PKEY_Device_Service,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 6}
DEFINE_PROPERTYKEY!{PKEY_Device_Class,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 9}
DEFINE_PROPERTYKEY!{PKEY_Device_ClassGuid,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 10}
DEFINE_PROPERTYKEY!{PKEY_Device_Driver,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 11}
DEFINE_PROPERTYKEY!{PKEY_Device_ConfigFlags,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 12}
DEFINE_PROPERTYKEY!{PKEY_Device_Manufacturer,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 13}
DEFINE_PROPERTYKEY!{PKEY_Device_FriendlyName,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 14}
DEFINE_PROPERTYKEY!{PKEY_Device_LocationInfo,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 15}
DEFINE_PROPERTYKEY!{PKEY_Device_PDOName,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 16}
DEFINE_PROPERTYKEY!{PKEY_Device_Capabilities,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 17}
DEFINE_PROPERTYKEY!{PKEY_Device_UINumber,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 18}
DEFINE_PROPERTYKEY!{PKEY_Device_UpperFilters,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 19}
DEFINE_PROPERTYKEY!{PKEY_Device_LowerFilters,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 20}
DEFINE_PROPERTYKEY!{PKEY_Device_BusTypeGuid,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 21}
DEFINE_PROPERTYKEY!{PKEY_Device_LegacyBusType,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 22}
DEFINE_PROPERTYKEY!{PKEY_Device_BusNumber,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 23}
DEFINE_PROPERTYKEY!{PKEY_Device_EnumeratorName,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 24}
DEFINE_PROPERTYKEY!{PKEY_Device_Security,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 25}
DEFINE_PROPERTYKEY!{PKEY_Device_SecuritySDS,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 26}
DEFINE_PROPERTYKEY!{PKEY_Device_DevType,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 27}
DEFINE_PROPERTYKEY!{PKEY_Device_Exclusive,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 28}
DEFINE_PROPERTYKEY!{PKEY_Device_Characteristics,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 29}
DEFINE_PROPERTYKEY!{PKEY_Device_Address,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 30}
DEFINE_PROPERTYKEY!{PKEY_Device_UINumberDescFormat,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 31}
DEFINE_PROPERTYKEY!{PKEY_Device_PowerData,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 32}
DEFINE_PROPERTYKEY!{PKEY_Device_RemovalPolicy,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 33}
DEFINE_PROPERTYKEY!{PKEY_Device_RemovalPolicyDefault,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 34}
DEFINE_PROPERTYKEY!{PKEY_Device_RemovalPolicyOverride,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 35}
DEFINE_PROPERTYKEY!{PKEY_Device_InstallState,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 36}
DEFINE_PROPERTYKEY!{PKEY_Device_LocationPaths,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 37}
DEFINE_PROPERTYKEY!{PKEY_Device_BaseContainerId,
    0xa45c254e, 0xdf1c, 0x4efd, 0x80, 0x20, 0x67, 0xd1, 0x46, 0xa8, 0x50, 0xe0, 38}
//137
DEFINE_PROPERTYKEY!{PKEY_Device_DriverDate,
    0xa8b865dd, 0x2e3d, 0x4094, 0xad, 0x97, 0xe5, 0x93, 0xa7, 0xc, 0x75, 0xd6, 2}
DEFINE_PROPERTYKEY!{PKEY_Device_DriverVersion,
    0xa8b865dd, 0x2e3d, 0x4094, 0xad, 0x97, 0xe5, 0x93, 0xa7, 0xc, 0x75, 0xd6, 3}
DEFINE_PROPERTYKEY!{PKEY_Device_DriverDesc,
    0xa8b865dd, 0x2e3d, 0x4094, 0xad, 0x97, 0xe5, 0x93, 0xa7, 0xc, 0x75, 0xd6, 4}
DEFINE_PROPERTYKEY!{PKEY_Device_DriverInfPath,
    0xa8b865dd, 0x2e3d, 0x4094, 0xad, 0x97, 0xe5, 0x93, 0xa7, 0xc, 0x75, 0xd6, 5}
DEFINE_PROPERTYKEY!{PKEY_Device_DriverInfSection,
    0xa8b865dd, 0x2e3d, 0x4094, 0xad, 0x97, 0xe5, 0x93, 0xa7, 0xc, 0x75, 0xd6, 6}
DEFINE_PROPERTYKEY!{PKEY_Device_DriverInfSectionExt,
    0xa8b865dd, 0x2e3d, 0x4094, 0xad, 0x97, 0xe5, 0x93, 0xa7, 0xc, 0x75, 0xd6, 7}
DEFINE_PROPERTYKEY!{PKEY_Device_MatchingDeviceId,
    0xa8b865dd, 0x2e3d, 0x4094, 0xad, 0x97, 0xe5, 0x93, 0xa7, 0xc, 0x75, 0xd6, 8}
DEFINE_PROPERTYKEY!{PKEY_Device_DriverProvider,
    0xa8b865dd, 0x2e3d, 0x4094, 0xad, 0x97, 0xe5, 0x93, 0xa7, 0xc, 0x75, 0xd6, 9}
DEFINE_PROPERTYKEY!{PKEY_Device_DriverPropPageProvider,
    0xa8b865dd, 0x2e3d, 0x4094, 0xad, 0x97, 0xe5, 0x93, 0xa7, 0xc, 0x75, 0xd6, 10}
DEFINE_PROPERTYKEY!{PKEY_Device_DriverCoInstallers,
    0xa8b865dd, 0x2e3d, 0x4094, 0xad, 0x97, 0xe5, 0x93, 0xa7, 0xc, 0x75, 0xd6, 11}
DEFINE_PROPERTYKEY!{PKEY_Device_ResourcePickerTags,
    0xa8b865dd, 0x2e3d, 0x4094, 0xad, 0x97, 0xe5, 0x93, 0xa7, 0xc, 0x75, 0xd6, 12}
DEFINE_PROPERTYKEY!{PKEY_Device_ResourcePickerExceptions,
    0xa8b865dd, 0x2e3d, 0x4094, 0xad, 0x97, 0xe5, 0x93, 0xa7, 0xc, 0x75, 0xd6, 13}
DEFINE_PROPERTYKEY!{PKEY_Device_DriverRank,
    0xa8b865dd, 0x2e3d, 0x4094, 0xad, 0x97, 0xe5, 0x93, 0xa7, 0xc, 0x75, 0xd6, 14}
DEFINE_PROPERTYKEY!{PKEY_Device_DriverLogoLevel,
    0xa8b865dd, 0x2e3d, 0x4094, 0xad, 0x97, 0xe5, 0x93, 0xa7, 0xc, 0x75, 0xd6, 15}
DEFINE_PROPERTYKEY!{PKEY_Device_NoConnectSound,
    0xa8b865dd, 0x2e3d, 0x4094, 0xad, 0x97, 0xe5, 0x93, 0xa7, 0xc, 0x75, 0xd6, 17}
DEFINE_PROPERTYKEY!{PKEY_Device_GenericDriverInstalled,
    0xa8b865dd, 0x2e3d, 0x4094, 0xad, 0x97, 0xe5, 0x93, 0xa7, 0xc, 0x75, 0xd6, 18}
DEFINE_PROPERTYKEY!{PKEY_Device_AdditionalSoftwareRequested,
    0xa8b865dd, 0x2e3d, 0x4094, 0xad, 0x97, 0xe5, 0x93, 0xa7, 0xc, 0x75, 0xd6, 19}
//209
DEFINE_PROPERTYKEY!{PKEY_DeviceInterface_FriendlyName,
    0x026e516e, 0xb814, 0x414b, 0x83, 0xcd, 0x85, 0x6d, 0x6f, 0xef, 0x48, 0x22, 2}
DEFINE_PROPERTYKEY!{PKEY_DeviceInterface_Enabled,
    0x026e516e, 0xb814, 0x414b, 0x83, 0xcd, 0x85, 0x6d, 0x6f, 0xef, 0x48, 0x22, 3}
DEFINE_PROPERTYKEY!{PKEY_DeviceInterface_ClassGuid,
    0x026e516e, 0xb814, 0x414b, 0x83, 0xcd, 0x85, 0x6d, 0x6f, 0xef, 0x48, 0x22, 4}
