::windows_targets::link ! ( "dmprocessxmlfiltered.dll""system" #[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"] fn DMProcessConfigXMLFiltered ( pszxmlin : ::windows_sys::core::PCWSTR , rgszallowedcspnodes : *const ::windows_sys::core::PCWSTR , dwnumallowedcspnodes : u32 , pbstrxmlout : *mut ::windows_sys::core::BSTR ) -> ::windows_sys::core::HRESULT );
pub type IConnectionRequestCallback = *mut ::core::ffi::c_void;
pub type IEnumPortableDeviceConnectors = *mut ::core::ffi::c_void;
pub type IEnumPortableDeviceObjectIDs = *mut ::core::ffi::c_void;
pub type IMediaRadioManager = *mut ::core::ffi::c_void;
pub type IMediaRadioManagerNotifySink = *mut ::core::ffi::c_void;
pub type IPortableDevice = *mut ::core::ffi::c_void;
pub type IPortableDeviceCapabilities = *mut ::core::ffi::c_void;
pub type IPortableDeviceConnector = *mut ::core::ffi::c_void;
pub type IPortableDeviceContent = *mut ::core::ffi::c_void;
pub type IPortableDeviceContent2 = *mut ::core::ffi::c_void;
pub type IPortableDeviceDataStream = *mut ::core::ffi::c_void;
pub type IPortableDeviceDispatchFactory = *mut ::core::ffi::c_void;
pub type IPortableDeviceEventCallback = *mut ::core::ffi::c_void;
pub type IPortableDeviceKeyCollection = *mut ::core::ffi::c_void;
pub type IPortableDeviceManager = *mut ::core::ffi::c_void;
pub type IPortableDevicePropVariantCollection = *mut ::core::ffi::c_void;
pub type IPortableDeviceProperties = *mut ::core::ffi::c_void;
pub type IPortableDevicePropertiesBulk = *mut ::core::ffi::c_void;
pub type IPortableDevicePropertiesBulkCallback = *mut ::core::ffi::c_void;
pub type IPortableDeviceResources = *mut ::core::ffi::c_void;
pub type IPortableDeviceService = *mut ::core::ffi::c_void;
pub type IPortableDeviceServiceActivation = *mut ::core::ffi::c_void;
pub type IPortableDeviceServiceCapabilities = *mut ::core::ffi::c_void;
pub type IPortableDeviceServiceManager = *mut ::core::ffi::c_void;
pub type IPortableDeviceServiceMethodCallback = *mut ::core::ffi::c_void;
pub type IPortableDeviceServiceMethods = *mut ::core::ffi::c_void;
pub type IPortableDeviceServiceOpenCallback = *mut ::core::ffi::c_void;
pub type IPortableDeviceUnitsStream = *mut ::core::ffi::c_void;
pub type IPortableDeviceValues = *mut ::core::ffi::c_void;
pub type IPortableDeviceValuesCollection = *mut ::core::ffi::c_void;
pub type IPortableDeviceWebControl = *mut ::core::ffi::c_void;
pub type IRadioInstance = *mut ::core::ffi::c_void;
pub type IRadioInstanceCollection = *mut ::core::ffi::c_void;
pub type IWpdSerializer = *mut ::core::ffi::c_void;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const CLSID_WPD_NAMESPACE_EXTENSION: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x35786d3c_b075_49b9_88dd_029876e11c01);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_Devices_Properties\"`*"]
#[cfg(feature = "Win32_Devices_Properties")]
pub const DEVPKEY_MTPBTH_IsConnected: super::Properties::DEVPROPKEY = super::Properties::DEVPROPKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xea1237fa_589d_4472_84e4_0abe36fd62ef), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const DEVSVCTYPE_ABSTRACT: u32 = 1u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const DEVSVCTYPE_DEFAULT: u32 = 0u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const DEVSVC_SERVICEINFO_VERSION: u32 = 100u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_AnchorResults_AnchorStateInvalid: u32 = 1u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_AnchorResults_AnchorStateNormal: u32 = 0u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_AnchorResults_AnchorStateOld: u32 = 2u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_AnchorResults_ItemStateChanged: u32 = 4u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_AnchorResults_ItemStateCreated: u32 = 2u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_AnchorResults_ItemStateDeleted: u32 = 1u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_AnchorResults_ItemStateInvalid: u32 = 0u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_AnchorResults_ItemStateUpdated: u32 = 3u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_CalendarObj_BusyStatusBusy: u32 = 1u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_CalendarObj_BusyStatusFree: u32 = 0u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_CalendarObj_BusyStatusOutOfOffice: u32 = 2u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_CalendarObj_BusyStatusTentative: u32 = 3u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_DeviceMetadataObj_DefaultCABFalse: u32 = 0u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_DeviceMetadataObj_DefaultCABTrue: u32 = 1u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_MessageObj_PatternInstanceFirst: u32 = 1u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_MessageObj_PatternInstanceFourth: u32 = 4u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_MessageObj_PatternInstanceLast: u32 = 5u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_MessageObj_PatternInstanceNone: u32 = 0u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_MessageObj_PatternInstanceSecond: u32 = 2u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_MessageObj_PatternInstanceThird: u32 = 3u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_MessageObj_PatternTypeDaily: u32 = 1u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_MessageObj_PatternTypeMonthly: u32 = 3u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_MessageObj_PatternTypeWeekly: u32 = 2u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_MessageObj_PatternTypeYearly: u32 = 4u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_MessageObj_PriorityHighest: u32 = 2u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_MessageObj_PriorityLowest: u32 = 0u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_MessageObj_PriorityNormal: u32 = 1u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_MessageObj_ReadFalse: u32 = 0u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_MessageObj_ReadTrue: u32 = 255u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_StatusSvc_ChargingActive: u32 = 1u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_StatusSvc_ChargingInactive: u32 = 0u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_StatusSvc_ChargingUnknown: u32 = 2u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_StatusSvc_RoamingActive: u32 = 1u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_StatusSvc_RoamingInactive: u32 = 0u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_StatusSvc_RoamingUnknown: u32 = 2u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_SyncSvc_SyncObjectReferencesDisabled: u32 = 0u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_SyncSvc_SyncObjectReferencesEnabled: u32 = 255u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_TaskObj_CompleteFalse: u32 = 0u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const ENUM_TaskObj_CompleteTrue: u32 = 255u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const E_WPD_DEVICE_ALREADY_OPENED: ::windows_sys::core::HRESULT = -2144731135i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const E_WPD_DEVICE_IS_HUNG: ::windows_sys::core::HRESULT = -2144731130i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const E_WPD_DEVICE_NOT_OPEN: ::windows_sys::core::HRESULT = -2144731134i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const E_WPD_OBJECT_ALREADY_ATTACHED_TO_DEVICE: ::windows_sys::core::HRESULT = -2144731133i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const E_WPD_OBJECT_ALREADY_ATTACHED_TO_SERVICE: ::windows_sys::core::HRESULT = -2144730934i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const E_WPD_OBJECT_NOT_ATTACHED_TO_DEVICE: ::windows_sys::core::HRESULT = -2144731132i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const E_WPD_OBJECT_NOT_ATTACHED_TO_SERVICE: ::windows_sys::core::HRESULT = -2144730933i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const E_WPD_OBJECT_NOT_COMMITED: ::windows_sys::core::HRESULT = -2144731131i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const E_WPD_SERVICE_ALREADY_OPENED: ::windows_sys::core::HRESULT = -2144730936i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const E_WPD_SERVICE_BAD_PARAMETER_ORDER: ::windows_sys::core::HRESULT = -2144730932i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const E_WPD_SERVICE_NOT_OPEN: ::windows_sys::core::HRESULT = -2144730935i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const E_WPD_SMS_INVALID_MESSAGE_BODY: ::windows_sys::core::HRESULT = -2144731035i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const E_WPD_SMS_INVALID_RECIPIENT: ::windows_sys::core::HRESULT = -2144731036i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const E_WPD_SMS_SERVICE_UNAVAILABLE: ::windows_sys::core::HRESULT = -2144731034i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const EnumBthMtpConnectors: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xa1570149_e645_4f43_8b0d_409b061db2fc);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const FACILITY_WPD: u32 = 42u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const FLAG_MessageObj_DayOfWeekFriday: u32 = 32u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const FLAG_MessageObj_DayOfWeekMonday: u32 = 2u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const FLAG_MessageObj_DayOfWeekNone: u32 = 0u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const FLAG_MessageObj_DayOfWeekSaturday: u32 = 64u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const FLAG_MessageObj_DayOfWeekSunday: u32 = 1u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const FLAG_MessageObj_DayOfWeekThursday: u32 = 16u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const FLAG_MessageObj_DayOfWeekTuesday: u32 = 4u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const FLAG_MessageObj_DayOfWeekWednesday: u32 = 8u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const GUID_DEVINTERFACE_WPD: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x6ac27878_a6fa_4155_ba85_f98f491d4f33);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const GUID_DEVINTERFACE_WPD_PRIVATE: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xba0c718f_4ded_49b7_bdd3_fabe28661211);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const GUID_DEVINTERFACE_WPD_SERVICE: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x9ef44f80_3d64_4246_a6aa_206f328d1edc);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const IOCTL_WPD_MESSAGE_READWRITE_ACCESS: u32 = 4243720u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const IOCTL_WPD_MESSAGE_READ_ACCESS: u32 = 4210952u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_3GPP2File: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("3GPP2File");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_3GPPFile: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("3GPPFile");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AACFile: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AACFile");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AIFFFile: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AIFFFile");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AMRFile: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AMRFile");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ASFFile: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("ASFFile");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ASXPlaylist: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("ASXPlaylist");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ATSCTSFile: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("ATSCTSFile");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AVCHDFile: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AVCHDFile");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AVIFile: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AVIFile");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AbstractActivity: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AbstractActivity");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AbstractActivityOccurrence: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AbstractActivityOccurrence");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AbstractAudioAlbum: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AbstractAudioAlbum");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AbstractAudioPlaylist: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AbstractAudioPlaylist");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AbstractAudioVideoAlbum: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AbstractAudioVideoAlbum");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AbstractChapteredProduction: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AbstractChapteredProduction");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AbstractContact: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AbstractContact");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AbstractContactGroup: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AbstractContactGroup");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AbstractDocument: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AbstractDocument");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AbstractImageAlbum: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AbstractImageAlbum");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AbstractMediacast: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AbstractMediacast");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AbstractMessage: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AbstractMessage");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AbstractMessageFolder: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AbstractMessageFolder");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AbstractMultimediaAlbum: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AbstractMultimediaAlbum");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AbstractNote: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AbstractNote");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AbstractTask: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AbstractTask");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AbstractVideoAlbum: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AbstractVideoAlbum");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AbstractVideoPlaylist: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AbstractVideoPlaylist");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AnchorResults: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AnchorResults");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AnchorResults_Anchor: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Anchor");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AnchorResults_AnchorState: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AnchorState");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AnchorResults_ResultObjectID: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("ResultObjectID");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AnchorSyncKnowledge: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AnchorSyncKnowledge");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AnchorSyncSvc: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AnchorSync");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AnchorSyncSvc_BeginSync: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("BeginSync");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AnchorSyncSvc_CurrentAnchor: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AnchorCurrentAnchor");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AnchorSyncSvc_EndSync: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("EndSync");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AnchorSyncSvc_FilterType: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("FilterType");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AnchorSyncSvc_GetChangesSinceAnchor: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("GetChangesSinceAnchor");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AnchorSyncSvc_KnowledgeObjectID: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AnchorKnowledgeObjectID");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AnchorSyncSvc_LastSyncProxyID: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AnchorLastSyncProxyID");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AnchorSyncSvc_LocalOnlyDelete: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("LocalOnlyDelete");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AnchorSyncSvc_ProviderVersion: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AnchorProviderVersion");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AnchorSyncSvc_ReplicaID: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AnchorReplicaID");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AnchorSyncSvc_SyncFormat: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("SyncFormat");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AnchorSyncSvc_VersionProps: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AnchorVersionProps");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_Association: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Association");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AudibleFile: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AudibleFile");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AudioObj_AudioBitDepth: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AudioBitDepth");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AudioObj_AudioBitRate: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AudioBitRate");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AudioObj_AudioBlockAlignment: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AudioBlockAlignment");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AudioObj_AudioFormatCode: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AudioFormatCode");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AudioObj_Channels: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Channels");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_AudioObj_Lyrics: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Lyrics");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_BMPImage: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("BMPImage");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_CIFFImage: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("CIFFImage");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_CalendarObj_Accepted: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Accepted");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_CalendarObj_BeginDateTime: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("BeginDateTime");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_CalendarObj_BusyStatus: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("BusyStatus");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_CalendarObj_Declined: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Declined");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_CalendarObj_EndDateTime: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("EndDateTime");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_CalendarObj_Location: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Location");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_CalendarObj_PatternDuration: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PatternDuration");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_CalendarObj_PatternStartTime: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PatternStartTime");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_CalendarObj_ReminderOffset: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("ReminderOffset");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_CalendarObj_Tentative: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Tentative");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_CalendarObj_TimeZone: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("TimeZone");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_CalendarSvc: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Calendar");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_CalendarSvc_SyncWindowEnd: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("SyncWindowEnd");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_CalendarSvc_SyncWindowStart: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("SyncWindowStart");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_AnniversaryDate: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AnniversaryDate");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_Assistant: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Assistant");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_Birthdate: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Birthdate");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_BusinessAddressCity: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("BusinessAddressCity");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_BusinessAddressCountry: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("BusinessAddressCountry");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_BusinessAddressFull: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("BusinessAddressFull");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_BusinessAddressLine2: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("BusinessAddressLine2");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_BusinessAddressPostalCode: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("BusinessAddressPostalCode");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_BusinessAddressRegion: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("BusinessAddressRegion");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_BusinessAddressStreet: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("BusinessAddressStreet");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_BusinessEmail: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("BusinessEmail");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_BusinessEmail2: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("BusinessEmail2");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_BusinessFax: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("BusinessFax");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_BusinessPhone: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("BusinessPhone");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_BusinessPhone2: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("BusinessPhone2");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_BusinessWebAddress: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("BusinessWebAddress");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_Children: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Children");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_Email: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Email");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_FamilyName: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("FamilyName");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_Fax: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Fax");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_GivenName: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("GivenName");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_IMAddress: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("IMAddress");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_IMAddress2: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("IMAddress2");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_IMAddress3: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("IMAddress3");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_MiddleNames: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("MiddleNames");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_MobilePhone: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("MobilePhone");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_MobilePhone2: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("MobilePhone2");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_Organization: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Organization");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_OtherAddressCity: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("OtherAddressCity");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_OtherAddressCountry: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("OtherAddressCountry");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_OtherAddressFull: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("OtherAddressFull");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_OtherAddressLine2: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("OtherAddressLine2");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_OtherAddressPostalCode: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("OtherAddressPostalCode");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_OtherAddressRegion: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("OtherAddressRegion");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_OtherAddressStreet: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("OtherAddressStreet");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_OtherEmail: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("OtherEmail");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_OtherPhone: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("OtherPhone");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_Pager: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Pager");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_PersonalAddressCity: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PersonalAddressCity");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_PersonalAddressCountry: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PersonalAddressCountry");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_PersonalAddressFull: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PersonalAddressFull");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_PersonalAddressLine2: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PersonalAddressLine2");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_PersonalAddressPostalCode: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PersonalAddressPostalCode");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_PersonalAddressRegion: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PersonalAddressRegion");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_PersonalAddressStreet: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PersonalAddressStreet");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_PersonalEmail: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PersonalEmail");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_PersonalEmail2: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PersonalEmail2");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_PersonalFax: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PersonalFax");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_PersonalPhone: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PersonalPhone");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_PersonalPhone2: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PersonalPhone2");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_PersonalWebAddress: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PersonalWebAddress");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_Phone: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Phone");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_PhoneticFamilyName: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PhoneticFamilyName");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_PhoneticGivenName: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PhoneticGivenName");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_PhoneticOrganization: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PhoneticOrganization");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_Ringtone: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Ringtone");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_Role: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Role");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_Spouse: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Spouse");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_Suffix: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Suffix");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_Title: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Title");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactObj_WebAddress: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("WebAddress");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactSvc_SyncWithPhoneOnly: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("FilterType");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ContactsSvc: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Contacts");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_DPOFDocument: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("DPOFDocument");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_DVBTSFile: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("DVBTSFile");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_DeviceExecutable: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("DeviceExecutable");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_DeviceMetadataCAB: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("DeviceMetadataCAB");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_DeviceMetadataObj_ContentID: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("ContentID");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_DeviceMetadataObj_DefaultCAB: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("DefaultCAB");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_DeviceMetadataSvc: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Metadata");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_DeviceScript: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("DeviceScript");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_EXIFImage: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("EXIFImage");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ExcelDocument: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("ExcelDocument");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_FLACFile: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("FLACFile");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_FirmwareFile: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("FirmwareFile");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_FlashPixImage: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("FlashPixImage");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_FullEnumSyncKnowledge: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("FullEnumSyncKnowledge");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_FullEnumSyncSvc: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("FullEnumSync");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_FullEnumSyncSvc_BeginSync: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("BeginSync");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_FullEnumSyncSvc_EndSync: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("EndSync");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_FullEnumSyncSvc_FilterType: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("FilterType");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_FullEnumSyncSvc_KnowledgeObjectID: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("FullEnumKnowledgeObjectID");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_FullEnumSyncSvc_LastSyncProxyID: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("FullEnumLastSyncProxyID");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_FullEnumSyncSvc_LocalOnlyDelete: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("LocalOnlyDelete");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_FullEnumSyncSvc_ProviderVersion: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("FullEnumProviderVersion");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_FullEnumSyncSvc_ReplicaID: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("FullEnumReplicaID");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_FullEnumSyncSvc_SyncFormat: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("SyncFormat");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_FullEnumSyncSvc_VersionProps: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("FullEnumVersionProps");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GIFImage: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("GIFImage");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_AllowedFolderContents: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AllowedFolderContents");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_AssociationDesc: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AssociationDesc");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_AssociationType: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AssociationType");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_Copyright: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Copyright");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_Corrupt: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Corrupt");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_DRMStatus: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("DRMStatus");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_DateAccessed: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("DateAccessed");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_DateAdded: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("DateAdded");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_DateAuthored: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("DateAuthored");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_DateCreated: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("DateCreated");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_DateModified: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("DateModified");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_DateRevised: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("DateRevised");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_Description: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Description");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_Hidden: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Hidden");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_Keywords: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Keywords");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_LanguageLocale: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("LanguageLocale");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_Name: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Name");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_NonConsumable: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("NonConsumable");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_ObjectFileName: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("ObjectFileName");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_ObjectFormat: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("ObjectFormat");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_ObjectID: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("ObjectID");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_ObjectSize: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("ObjectSize");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_ParentID: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("ParentID");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_PersistentUID: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PersistentUID");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_PropertyBag: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PropertyBag");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_ProtectionStatus: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("ProtectionStatus");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_ReferenceParentID: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("ReferenceParentID");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_StorageID: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("StorageID");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_SubDescription: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("SubDescription");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_SyncID: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("SyncID");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_SystemObject: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("SystemObject");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_GenericObj_TimeToLive: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("TimeToLive");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_HDPhotoImage: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("HDPhotoImage");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_HTMLDocument: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("HTMLDocument");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_HintsSvc: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Hints");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ICalendarActivity: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("ICalendar");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ImageObj_Aperature: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Aperature");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ImageObj_Exposure: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Exposure");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ImageObj_ISOSpeed: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("ISOSpeed");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ImageObj_ImageBitDepth: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("ImageBitDepth");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ImageObj_IsColorCorrected: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("IsColorCorrected");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_ImageObj_IsCropped: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("IsCropped");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_JFIFImage: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("JFIFImage");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_JP2Image: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("JP2Image");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_JPEGXRImage: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("JPEGXRImage");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_JPXImage: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("JPXImage");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_M3UPlaylist: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("M3UPlaylist");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MHTDocument: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("MHTDocument");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MP3File: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("MP3File");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MPEG2File: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("MPEG2File");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MPEG4File: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("MPEG4File");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MPEGFile: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("MPEGFile");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MPLPlaylist: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("MPLPlaylist");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_AlbumArtist: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AlbumArtist");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_AlbumName: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AlbumName");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_Artist: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Artist");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_AudioEncodingProfile: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("AudioEncodingProfile");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_BitRateType: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("BitRateType");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_BookmarkByte: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("BookmarkByte");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_BookmarkObject: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("BookmarkObject");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_BookmarkTime: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("BookmarkTime");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_BufferSize: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("BufferSize");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_Composer: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Composer");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_Credits: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Credits");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_DateOriginalRelease: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("DateOriginalRelease");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_Duration: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Duration");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_Editor: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Editor");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_EffectiveRating: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("EffectiveRating");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_EncodingProfile: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("EncodingProfile");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_EncodingQuality: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("EncodingQuality");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_Genre: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Genre");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_GeographicOrigin: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("GeographicOrigin");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_Height: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Height");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_MediaType: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("MediaType");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_MediaUID: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("MediaUID");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_Mood: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Mood");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_Owner: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Owner");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_ParentalRating: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("ParentalRating");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_Producer: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Producer");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_SampleRate: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("SampleRate");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_SkipCount: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("SkipCount");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_SubscriptionContentID: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("SubscriptionContentID");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_Subtitle: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Subtitle");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_TotalBitRate: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("TotalBitRate");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_Track: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Track");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_URLLink: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("URLLink");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_URLSource: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("URLSource");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_UseCount: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("UseCount");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_UserRating: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("UserRating");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_WebMaster: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("WebMaster");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MediaObj_Width: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Width");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MessageObj_BCC: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("BCC");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MessageObj_Body: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Body");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MessageObj_CC: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("CC");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MessageObj_Category: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Category");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MessageObj_PatternDayOfMonth: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PatternDayOfMonth");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MessageObj_PatternDayOfWeek: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PatternDayOfWeek");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MessageObj_PatternDeleteDates: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PatternDeleteDates");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MessageObj_PatternInstance: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PatternInstance");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MessageObj_PatternMonthOfYear: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PatternMonthOfYear");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MessageObj_PatternOriginalDateTime: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PatternOriginalDateTime");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MessageObj_PatternPeriod: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PatternPeriod");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MessageObj_PatternType: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PatternType");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MessageObj_PatternValidEndDate: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PatternValidEndDate");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MessageObj_PatternValidStartDate: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PatternValidStartDate");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MessageObj_Priority: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Priority");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MessageObj_Read: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Read");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MessageObj_ReceivedTime: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("ReceivedTime");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MessageObj_Sender: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Sender");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MessageObj_Subject: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Subject");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MessageObj_To: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("To");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_MessageSvc: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Message");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_NotesSvc: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Notes");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_OGGFile: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("OGGFile");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_PCDImage: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PCDImage");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_PICTImage: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PICTImage");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_PNGImage: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PNGImage");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_PSLPlaylist: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PSLPlaylist");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_PowerPointDocument: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PowerPointDocument");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_QCELPFile: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("QCELPFile");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_RingtonesSvc: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Ringtones");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_RingtonesSvc_DefaultRingtone: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("DefaultRingtone");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_Services_ServiceDisplayName: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("ServiceDisplayName");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_Services_ServiceIcon: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("ServiceIcon");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_Services_ServiceLocale: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("ServiceLocale");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_StatusSvc: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Status");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_StatusSvc_BatteryLife: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("BatteryLife");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_StatusSvc_ChargingState: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("ChargingState");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_StatusSvc_MissedCalls: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("MissedCalls");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_StatusSvc_NetworkName: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("NetworkName");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_StatusSvc_NetworkType: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("NetworkType");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_StatusSvc_NewPictures: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("NewPictures");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_StatusSvc_Roaming: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Roaming");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_StatusSvc_SignalStrength: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("SignalStrength");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_StatusSvc_StorageCapacity: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("StorageCapacity");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_StatusSvc_StorageFreeSpace: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("StorageFreeSpace");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_StatusSvc_TextMessages: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("TextMessages");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_StatusSvc_VoiceMail: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("VoiceMail");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_SyncObj_LastAuthorProxyID: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("LastAuthorProxyID");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_SyncSvc_BeginSync: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("BeginSync");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_SyncSvc_EndSync: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("EndSync");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_SyncSvc_FilterType: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("FilterType");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_SyncSvc_LocalOnlyDelete: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("LocalOnlyDelete");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_SyncSvc_SyncFormat: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("SyncFormat");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_SyncSvc_SyncObjectReferences: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("SyncObjectReferences");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_TIFFEPImage: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("TIFFEPImage");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_TIFFITImage: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("TIFFITImage");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_TIFFImage: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("TIFFImage");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_TaskObj_BeginDate: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("BeginDate");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_TaskObj_Complete: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Complete");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_TaskObj_EndDate: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("EndDate");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_TaskObj_ReminderDateTime: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("ReminderDateTime");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_TasksSvc: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Tasks");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_TasksSvc_SyncActiveOnly: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("FilterType");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_TextDocument: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("TextDocument");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_Undefined: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Undefined");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_UndefinedAudio: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("UndefinedAudio");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_UndefinedCollection: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("UndefinedCollection");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_UndefinedDocument: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("UndefinedDocument");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_UndefinedVideo: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("UndefinedVideo");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_UnknownImage: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("UnknownImage");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_VCalendar1Activity: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("VCalendar1");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_VCard2Contact: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("VCard2Contact");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_VCard3Contact: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("VCard3Contact");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_VideoObj_KeyFrameDistance: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("KeyFrameDistance");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_VideoObj_ScanType: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("ScanType");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_VideoObj_Source: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Source");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_VideoObj_VideoBitRate: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("VideoBitRate");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_VideoObj_VideoFormatCode: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("VideoFormatCode");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_VideoObj_VideoFrameRate: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("VideoFrameRate");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_WAVFile: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("WAVFile");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_WBMPImage: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("WBMPImage");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_WMAFile: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("WMAFile");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_WMVFile: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("WMVFile");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_WPLPlaylist: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("WPLPlaylist");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_WordDocument: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("WordDocument");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const NAME_XMLDocument: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("XMLDocument");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const PORTABLE_DEVICE_DRM_SCHEME_PDDRM: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PDDRM");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const PORTABLE_DEVICE_DRM_SCHEME_WMDRM10_PD: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("WMDRM10-PD");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const PORTABLE_DEVICE_ICON: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("Icons");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const PORTABLE_DEVICE_IS_MASS_STORAGE: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PortableDeviceIsMassStorage");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const PORTABLE_DEVICE_NAMESPACE_EXCLUDE_FROM_SHELL: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PortableDeviceNameSpaceExcludeFromShell");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const PORTABLE_DEVICE_NAMESPACE_THUMBNAIL_CONTENT_TYPES: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PortableDeviceNameSpaceThumbnailContentTypes");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const PORTABLE_DEVICE_NAMESPACE_TIMEOUT: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PortableDeviceNameSpaceTimeout");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const PORTABLE_DEVICE_TYPE: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("PortableDeviceType");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const PortableDevice: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x728a21c5_3d9e_48d7_9810_864848f0f404);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const PortableDeviceDispatchFactory: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x43232233_8338_4658_ae01_0b4ae830b6b0);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const PortableDeviceFTM: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xf7c0039a_4762_488a_b4b3_760ef9a1ba9b);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const PortableDeviceKeyCollection: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xde2d022d_2480_43be_97f0_d1fa2cf98f4f);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const PortableDeviceManager: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x0af10cec_2ecd_4b92_9581_34f6ae0637f3);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const PortableDevicePropVariantCollection: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x08a99e2f_6d6d_4b80_af5a_baf2bcbe4cb9);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const PortableDeviceService: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xef5db4c2_9312_422c_9152_411cd9c4dd84);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const PortableDeviceServiceFTM: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x1649b154_c794_497a_9b03_f3f0121302f3);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const PortableDeviceValues: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x0c15d503_d017_47ce_9016_7b3f978721cc);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const PortableDeviceValuesCollection: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x3882134d_14cf_4220_9cb4_435f86d83f60);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const PortableDeviceWebControl: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x186dd02c_2dec_41b5_a7d4_b59056fade51);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const RANGEMAX_MessageObj_PatternDayOfMonth: u32 = 31u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const RANGEMAX_MessageObj_PatternMonthOfYear: u32 = 12u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const RANGEMAX_StatusSvc_BatteryLife: u32 = 100u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const RANGEMAX_StatusSvc_MissedCalls: u32 = 255u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const RANGEMAX_StatusSvc_NewPictures: u32 = 65535u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const RANGEMAX_StatusSvc_SignalStrength: u32 = 4u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const RANGEMAX_StatusSvc_TextMessages: u32 = 255u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const RANGEMAX_StatusSvc_VoiceMail: u32 = 255u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const RANGEMIN_MessageObj_PatternDayOfMonth: u32 = 1u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const RANGEMIN_MessageObj_PatternMonthOfYear: u32 = 1u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const RANGEMIN_StatusSvc_BatteryLife: u32 = 0u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const RANGEMIN_StatusSvc_SignalStrength: u32 = 0u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const RANGESTEP_MessageObj_PatternDayOfMonth: u32 = 1u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const RANGESTEP_MessageObj_PatternMonthOfYear: u32 = 1u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const RANGESTEP_StatusSvc_BatteryLife: u32 = 1u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const RANGESTEP_StatusSvc_SignalStrength: u32 = 1u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const STR_WPDNSE_FAST_ENUM: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("WPDNSE Fast Enum");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const STR_WPDNSE_SIMPLE_ITEM: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("WPDNSE SimpleItem");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const SYNCSVC_FILTER_CALENDAR_WINDOW_WITH_RECURRENCE: u32 = 3u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const SYNCSVC_FILTER_CONTACTS_WITH_PHONE: u32 = 1u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const SYNCSVC_FILTER_NONE: u32 = 0u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const SYNCSVC_FILTER_TASK_ACTIVE: u32 = 2u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const TYPE_AnchorSyncSvc: u32 = 1u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const TYPE_CalendarSvc: u32 = 0u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const TYPE_ContactsSvc: u32 = 0u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const TYPE_DeviceMetadataSvc: u32 = 0u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const TYPE_FullEnumSyncSvc: u32 = 1u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const TYPE_HintsSvc: u32 = 0u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const TYPE_MessageSvc: u32 = 0u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const TYPE_NotesSvc: u32 = 0u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const TYPE_RingtonesSvc: u32 = 0u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const TYPE_StatusSvc: u32 = 0u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const TYPE_TasksSvc: u32 = 0u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPDNSE_OBJECT_HAS_ALBUM_ART: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x34d71409_4b47_4d80_aaac_3a28a4a3b3e6), pid: 6 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPDNSE_OBJECT_HAS_AUDIO_CLIP: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x34d71409_4b47_4d80_aaac_3a28a4a3b3e6), pid: 5 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPDNSE_OBJECT_HAS_CONTACT_PHOTO: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x34d71409_4b47_4d80_aaac_3a28a4a3b3e6), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPDNSE_OBJECT_HAS_ICON: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x34d71409_4b47_4d80_aaac_3a28a4a3b3e6), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPDNSE_OBJECT_HAS_THUMBNAIL: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x34d71409_4b47_4d80_aaac_3a28a4a3b3e6), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPDNSE_OBJECT_OPTIMAL_READ_BLOCK_SIZE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x34d71409_4b47_4d80_aaac_3a28a4a3b3e6), pid: 7 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPDNSE_OBJECT_PROPERTIES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x34d71409_4b47_4d80_aaac_3a28a4a3b3e6);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPDNSE_PROPSHEET_CONTENT_DETAILS: u32 = 32u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPDNSE_PROPSHEET_CONTENT_GENERAL: u32 = 4u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPDNSE_PROPSHEET_CONTENT_REFERENCES: u32 = 8u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPDNSE_PROPSHEET_CONTENT_RESOURCES: u32 = 16u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPDNSE_PROPSHEET_DEVICE_GENERAL: u32 = 1u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPDNSE_PROPSHEET_STORAGE_GENERAL: u32 = 2u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_API_OPTIONS_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x10e54a3e_052d_4777_a13c_de7614be2bc4);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_API_OPTION_IOCTL_ACCESS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x10e54a3e_052d_4777_a13c_de7614be2bc4), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_API_OPTION_USE_CLEAR_DATA_STREAM: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x10e54a3e_052d_4777_a13c_de7614be2bc4), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_APPOINTMENT_ACCEPTED_ATTENDEES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xf99efd03_431d_40d8_a1c9_4e220d9c88d3), pid: 10 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_APPOINTMENT_DECLINED_ATTENDEES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xf99efd03_431d_40d8_a1c9_4e220d9c88d3), pid: 13 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_APPOINTMENT_LOCATION: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xf99efd03_431d_40d8_a1c9_4e220d9c88d3), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_APPOINTMENT_OBJECT_PROPERTIES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xf99efd03_431d_40d8_a1c9_4e220d9c88d3);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_APPOINTMENT_OPTIONAL_ATTENDEES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xf99efd03_431d_40d8_a1c9_4e220d9c88d3), pid: 9 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_APPOINTMENT_REQUIRED_ATTENDEES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xf99efd03_431d_40d8_a1c9_4e220d9c88d3), pid: 8 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_APPOINTMENT_RESOURCES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xf99efd03_431d_40d8_a1c9_4e220d9c88d3), pid: 11 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_APPOINTMENT_TENTATIVE_ATTENDEES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xf99efd03_431d_40d8_a1c9_4e220d9c88d3), pid: 12 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_APPOINTMENT_TYPE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xf99efd03_431d_40d8_a1c9_4e220d9c88d3), pid: 7 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_AUDIO_BITRATE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb324f56a_dc5d_46e5_b6df_d2ea414888c6), pid: 9 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_AUDIO_BIT_DEPTH: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb324f56a_dc5d_46e5_b6df_d2ea414888c6), pid: 12 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_AUDIO_BLOCK_ALIGNMENT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb324f56a_dc5d_46e5_b6df_d2ea414888c6), pid: 13 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_AUDIO_CHANNEL_COUNT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb324f56a_dc5d_46e5_b6df_d2ea414888c6), pid: 10 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_AUDIO_FORMAT_CODE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb324f56a_dc5d_46e5_b6df_d2ea414888c6), pid: 11 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CATEGORY_CAPABILITIES: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x0cabec78_6b74_41c6_9216_2639d1fce356);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CATEGORY_COMMON: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xf0422a9c_5dc8_4440_b5bd_5df28835658a);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CATEGORY_DEVICE_HINTS: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x0d5fb92b_cb46_4c4f_8343_0bc3d3f17c84);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CATEGORY_MEDIA_CAPTURE: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x59b433ba_fe44_4d8d_808c_6bcb9b0f15e8);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CATEGORY_MTP_EXT_VENDOR_OPERATIONS: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x4d545058_1a2e_4106_a357_771e0819fc56);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CATEGORY_NETWORK_CONFIGURATION: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x78f9c6fc_79b8_473c_9060_6bd23dd072c4);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CATEGORY_NULL: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x00000000_0000_0000_0000_000000000000);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CATEGORY_OBJECT_ENUMERATION: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xb7474e91_e7f8_4ad9_b400_ad1a4b58eeec);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CATEGORY_OBJECT_MANAGEMENT: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xef1e43dd_a9ed_4341_8bcc_186192aea089);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CATEGORY_OBJECT_PROPERTIES: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x9e5582e4_0814_44e6_981a_b2998d583804);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CATEGORY_OBJECT_PROPERTIES_BULK: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x11c824dd_04cd_4e4e_8c7b_f6efb794d84e);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CATEGORY_OBJECT_RESOURCES: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CATEGORY_SERVICE_CAPABILITIES: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CATEGORY_SERVICE_COMMON: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x322f071d_36ef_477f_b4b5_6f52d734baee);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CATEGORY_SERVICE_METHODS: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x2d521ca8_c1b0_4268_a342_cf19321569bc);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CATEGORY_SMS: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xafc25d66_fe0d_4114_9097_970c93e920d1);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CATEGORY_STILL_IMAGE_CAPTURE: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x4fcd6982_22a2_4b05_a48b_62d38bf27b32);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CATEGORY_STORAGE: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xd8f907a6_34cc_45fa_97fb_d007fa47ec94);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CLASS_EXTENSION_OPTIONS_DEVICE_IDENTIFICATION_VALUES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x3e3595da_4d71_49fe_a0b4_d4406c3ae93f), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CLASS_EXTENSION_OPTIONS_DONT_REGISTER_WPD_DEVICE_INTERFACE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x6309ffef_a87c_4ca7_8434_797576e40a96), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CLASS_EXTENSION_OPTIONS_MULTITRANSPORT_MODE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x3e3595da_4d71_49fe_a0b4_d4406c3ae93f), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CLASS_EXTENSION_OPTIONS_REGISTER_WPD_PRIVATE_DEVICE_INTERFACE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x6309ffef_a87c_4ca7_8434_797576e40a96), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CLASS_EXTENSION_OPTIONS_SILENCE_AUTOPLAY: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x65c160f8_1367_4ce2_939d_8310839f0d30), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CLASS_EXTENSION_OPTIONS_SUPPORTED_CONTENT_TYPES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x6309ffef_a87c_4ca7_8434_797576e40a96), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CLASS_EXTENSION_OPTIONS_TRANSPORT_BANDWIDTH: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x3e3595da_4d71_49fe_a0b4_d4406c3ae93f), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CLASS_EXTENSION_OPTIONS_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x6309ffef_a87c_4ca7_8434_797576e40a96);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CLASS_EXTENSION_OPTIONS_V2: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x3e3595da_4d71_49fe_a0b4_d4406c3ae93f);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CLASS_EXTENSION_OPTIONS_V3: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x65c160f8_1367_4ce2_939d_8310839f0d30);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CLASS_EXTENSION_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x33fb0d11_64a3_4fac_b4c7_3dfeaa99b051);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CLASS_EXTENSION_V2: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x7f0779b5_fa2b_4766_9cb2_f73ba30b6758);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CLIENT_DESIRED_ACCESS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x204d9f0c_2292_4080_9f42_40664e70f859), pid: 9 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CLIENT_EVENT_COOKIE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x204d9f0c_2292_4080_9f42_40664e70f859), pid: 11 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CLIENT_INFORMATION_PROPERTIES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x204d9f0c_2292_4080_9f42_40664e70f859);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CLIENT_MAJOR_VERSION: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x204d9f0c_2292_4080_9f42_40664e70f859), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CLIENT_MANUAL_CLOSE_ON_DISCONNECT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x204d9f0c_2292_4080_9f42_40664e70f859), pid: 13 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CLIENT_MINIMUM_RESULTS_BUFFER_SIZE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x204d9f0c_2292_4080_9f42_40664e70f859), pid: 12 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CLIENT_MINOR_VERSION: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x204d9f0c_2292_4080_9f42_40664e70f859), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CLIENT_NAME: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x204d9f0c_2292_4080_9f42_40664e70f859), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CLIENT_REVISION: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x204d9f0c_2292_4080_9f42_40664e70f859), pid: 5 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CLIENT_SECURITY_QUALITY_OF_SERVICE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x204d9f0c_2292_4080_9f42_40664e70f859), pid: 8 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CLIENT_SHARE_MODE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x204d9f0c_2292_4080_9f42_40664e70f859), pid: 10 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CLIENT_WMDRM_APPLICATION_CERTIFICATE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x204d9f0c_2292_4080_9f42_40664e70f859), pid: 7 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CLIENT_WMDRM_APPLICATION_PRIVATE_KEY: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x204d9f0c_2292_4080_9f42_40664e70f859), pid: 6 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_CAPABILITIES_GET_COMMAND_OPTIONS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x0cabec78_6b74_41c6_9216_2639d1fce356), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_CAPABILITIES_GET_EVENT_OPTIONS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x0cabec78_6b74_41c6_9216_2639d1fce356), pid: 11 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_CAPABILITIES_GET_FIXED_PROPERTY_ATTRIBUTES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x0cabec78_6b74_41c6_9216_2639d1fce356), pid: 9 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_CAPABILITIES_GET_FUNCTIONAL_OBJECTS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x0cabec78_6b74_41c6_9216_2639d1fce356), pid: 5 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_CAPABILITIES_GET_SUPPORTED_COMMANDS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x0cabec78_6b74_41c6_9216_2639d1fce356), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_CAPABILITIES_GET_SUPPORTED_CONTENT_TYPES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x0cabec78_6b74_41c6_9216_2639d1fce356), pid: 6 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_CAPABILITIES_GET_SUPPORTED_EVENTS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x0cabec78_6b74_41c6_9216_2639d1fce356), pid: 10 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_CAPABILITIES_GET_SUPPORTED_FORMATS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x0cabec78_6b74_41c6_9216_2639d1fce356), pid: 7 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_CAPABILITIES_GET_SUPPORTED_FORMAT_PROPERTIES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x0cabec78_6b74_41c6_9216_2639d1fce356), pid: 8 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_CAPABILITIES_GET_SUPPORTED_FUNCTIONAL_CATEGORIES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x0cabec78_6b74_41c6_9216_2639d1fce356), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_CLASS_EXTENSION_REGISTER_SERVICE_INTERFACES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x7f0779b5_fa2b_4766_9cb2_f73ba30b6758), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_CLASS_EXTENSION_UNREGISTER_SERVICE_INTERFACES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x7f0779b5_fa2b_4766_9cb2_f73ba30b6758), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_CLASS_EXTENSION_WRITE_DEVICE_INFORMATION: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x33fb0d11_64a3_4fac_b4c7_3dfeaa99b051), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_COMMIT_KEYPAIR: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x78f9c6fc_79b8_473c_9060_6bd23dd072c4), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_COMMON_GET_OBJECT_IDS_FROM_PERSISTENT_UNIQUE_IDS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xf0422a9c_5dc8_4440_b5bd_5df28835658a), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_COMMON_RESET_DEVICE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xf0422a9c_5dc8_4440_b5bd_5df28835658a), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_COMMON_SAVE_CLIENT_INFORMATION: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xf0422a9c_5dc8_4440_b5bd_5df28835658a), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_DEVICE_HINTS_GET_CONTENT_LOCATION: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x0d5fb92b_cb46_4c4f_8343_0bc3d3f17c84), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_GENERATE_KEYPAIR: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x78f9c6fc_79b8_473c_9060_6bd23dd072c4), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_MEDIA_CAPTURE_PAUSE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x59b433ba_fe44_4d8d_808c_6bcb9b0f15e8), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_MEDIA_CAPTURE_START: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x59b433ba_fe44_4d8d_808c_6bcb9b0f15e8), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_MEDIA_CAPTURE_STOP: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x59b433ba_fe44_4d8d_808c_6bcb9b0f15e8), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_MTP_EXT_END_DATA_TRANSFER: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x4d545058_1a2e_4106_a357_771e0819fc56), pid: 17 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_MTP_EXT_EXECUTE_COMMAND_WITHOUT_DATA_PHASE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x4d545058_1a2e_4106_a357_771e0819fc56), pid: 12 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_MTP_EXT_EXECUTE_COMMAND_WITH_DATA_TO_READ: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x4d545058_1a2e_4106_a357_771e0819fc56), pid: 13 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_MTP_EXT_EXECUTE_COMMAND_WITH_DATA_TO_WRITE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x4d545058_1a2e_4106_a357_771e0819fc56), pid: 14 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_MTP_EXT_GET_SUPPORTED_VENDOR_OPCODES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x4d545058_1a2e_4106_a357_771e0819fc56), pid: 11 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_MTP_EXT_GET_VENDOR_EXTENSION_DESCRIPTION: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x4d545058_1a2e_4106_a357_771e0819fc56), pid: 18 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_MTP_EXT_READ_DATA: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x4d545058_1a2e_4106_a357_771e0819fc56), pid: 15 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_MTP_EXT_WRITE_DATA: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x4d545058_1a2e_4106_a357_771e0819fc56), pid: 16 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_ENUMERATION_END_FIND: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb7474e91_e7f8_4ad9_b400_ad1a4b58eeec), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_ENUMERATION_FIND_NEXT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb7474e91_e7f8_4ad9_b400_ad1a4b58eeec), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_ENUMERATION_START_FIND: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb7474e91_e7f8_4ad9_b400_ad1a4b58eeec), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_MANAGEMENT_COMMIT_OBJECT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef1e43dd_a9ed_4341_8bcc_186192aea089), pid: 5 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_MANAGEMENT_COPY_OBJECTS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef1e43dd_a9ed_4341_8bcc_186192aea089), pid: 9 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_MANAGEMENT_CREATE_OBJECT_WITH_PROPERTIES_AND_DATA: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef1e43dd_a9ed_4341_8bcc_186192aea089), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_MANAGEMENT_CREATE_OBJECT_WITH_PROPERTIES_ONLY: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef1e43dd_a9ed_4341_8bcc_186192aea089), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_MANAGEMENT_DELETE_OBJECTS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef1e43dd_a9ed_4341_8bcc_186192aea089), pid: 7 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_MANAGEMENT_MOVE_OBJECTS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef1e43dd_a9ed_4341_8bcc_186192aea089), pid: 8 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_MANAGEMENT_REVERT_OBJECT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef1e43dd_a9ed_4341_8bcc_186192aea089), pid: 6 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_MANAGEMENT_UPDATE_OBJECT_WITH_PROPERTIES_AND_DATA: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef1e43dd_a9ed_4341_8bcc_186192aea089), pid: 10 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_MANAGEMENT_WRITE_OBJECT_DATA: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef1e43dd_a9ed_4341_8bcc_186192aea089), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_PROPERTIES_BULK_GET_VALUES_BY_OBJECT_FORMAT_END: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x11c824dd_04cd_4e4e_8c7b_f6efb794d84e), pid: 7 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_PROPERTIES_BULK_GET_VALUES_BY_OBJECT_FORMAT_NEXT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x11c824dd_04cd_4e4e_8c7b_f6efb794d84e), pid: 6 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_PROPERTIES_BULK_GET_VALUES_BY_OBJECT_FORMAT_START: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x11c824dd_04cd_4e4e_8c7b_f6efb794d84e), pid: 5 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_PROPERTIES_BULK_GET_VALUES_BY_OBJECT_LIST_END: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x11c824dd_04cd_4e4e_8c7b_f6efb794d84e), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_PROPERTIES_BULK_GET_VALUES_BY_OBJECT_LIST_NEXT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x11c824dd_04cd_4e4e_8c7b_f6efb794d84e), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_PROPERTIES_BULK_GET_VALUES_BY_OBJECT_LIST_START: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x11c824dd_04cd_4e4e_8c7b_f6efb794d84e), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_PROPERTIES_BULK_SET_VALUES_BY_OBJECT_LIST_END: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x11c824dd_04cd_4e4e_8c7b_f6efb794d84e), pid: 10 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_PROPERTIES_BULK_SET_VALUES_BY_OBJECT_LIST_NEXT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x11c824dd_04cd_4e4e_8c7b_f6efb794d84e), pid: 9 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_PROPERTIES_BULK_SET_VALUES_BY_OBJECT_LIST_START: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x11c824dd_04cd_4e4e_8c7b_f6efb794d84e), pid: 8 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_PROPERTIES_DELETE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x9e5582e4_0814_44e6_981a_b2998d583804), pid: 7 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_PROPERTIES_GET: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x9e5582e4_0814_44e6_981a_b2998d583804), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_PROPERTIES_GET_ALL: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x9e5582e4_0814_44e6_981a_b2998d583804), pid: 6 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_PROPERTIES_GET_ATTRIBUTES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x9e5582e4_0814_44e6_981a_b2998d583804), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_PROPERTIES_GET_SUPPORTED: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x9e5582e4_0814_44e6_981a_b2998d583804), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_PROPERTIES_SET: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x9e5582e4_0814_44e6_981a_b2998d583804), pid: 5 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_RESOURCES_CLOSE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 7 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_RESOURCES_COMMIT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 12 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_RESOURCES_CREATE_RESOURCE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 9 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_RESOURCES_DELETE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 8 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_RESOURCES_GET_ATTRIBUTES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_RESOURCES_GET_SUPPORTED: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_RESOURCES_OPEN: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_RESOURCES_READ: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 5 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_RESOURCES_REVERT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 10 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_RESOURCES_SEEK: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 11 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_RESOURCES_SEEK_IN_UNITS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 13 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_OBJECT_RESOURCES_WRITE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 6 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_PROCESS_WIRELESS_PROFILE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x78f9c6fc_79b8_473c_9060_6bd23dd072c4), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_SERVICE_CAPABILITIES_GET_COMMAND_OPTIONS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 16 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_SERVICE_CAPABILITIES_GET_EVENT_ATTRIBUTES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 11 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_SERVICE_CAPABILITIES_GET_EVENT_PARAMETER_ATTRIBUTES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 12 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_SERVICE_CAPABILITIES_GET_FORMAT_ATTRIBUTES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 7 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_SERVICE_CAPABILITIES_GET_FORMAT_PROPERTY_ATTRIBUTES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 9 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_SERVICE_CAPABILITIES_GET_FORMAT_RENDERING_PROFILES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 14 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_SERVICE_CAPABILITIES_GET_INHERITED_SERVICES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 13 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_SERVICE_CAPABILITIES_GET_METHOD_ATTRIBUTES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_SERVICE_CAPABILITIES_GET_METHOD_PARAMETER_ATTRIBUTES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 5 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_SERVICE_CAPABILITIES_GET_SUPPORTED_COMMANDS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 15 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_SERVICE_CAPABILITIES_GET_SUPPORTED_EVENTS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 10 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_SERVICE_CAPABILITIES_GET_SUPPORTED_FORMATS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 6 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_SERVICE_CAPABILITIES_GET_SUPPORTED_FORMAT_PROPERTIES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 8 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_SERVICE_CAPABILITIES_GET_SUPPORTED_METHODS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_SERVICE_CAPABILITIES_GET_SUPPORTED_METHODS_BY_FORMAT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_SERVICE_COMMON_GET_SERVICE_OBJECT_ID: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x322f071d_36ef_477f_b4b5_6f52d734baee), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_SERVICE_METHODS_CANCEL_INVOKE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2d521ca8_c1b0_4268_a342_cf19321569bc), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_SERVICE_METHODS_END_INVOKE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2d521ca8_c1b0_4268_a342_cf19321569bc), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_SERVICE_METHODS_START_INVOKE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2d521ca8_c1b0_4268_a342_cf19321569bc), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_SMS_SEND: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xafc25d66_fe0d_4114_9097_970c93e920d1), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_STILL_IMAGE_CAPTURE_INITIATE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x4fcd6982_22a2_4b05_a48b_62d38bf27b32), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_STORAGE_EJECT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xd8f907a6_34cc_45fa_97fb_d007fa47ec94), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMAND_STORAGE_FORMAT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xd8f907a6_34cc_45fa_97fb_d007fa47ec94), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMON_INFORMATION_BODY_TEXT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb28ae94b_05a4_4e8e_be01_72cc7e099d8f), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMON_INFORMATION_END_DATETIME: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb28ae94b_05a4_4e8e_be01_72cc7e099d8f), pid: 6 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMON_INFORMATION_NOTES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb28ae94b_05a4_4e8e_be01_72cc7e099d8f), pid: 7 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_COMMON_INFORMATION_OBJECT_PROPERTIES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xb28ae94b_05a4_4e8e_be01_72cc7e099d8f);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMON_INFORMATION_PRIORITY: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb28ae94b_05a4_4e8e_be01_72cc7e099d8f), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMON_INFORMATION_START_DATETIME: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb28ae94b_05a4_4e8e_be01_72cc7e099d8f), pid: 5 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_COMMON_INFORMATION_SUBJECT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb28ae94b_05a4_4e8e_be01_72cc7e099d8f), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_ANNIVERSARY_DATE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 62 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_ASSISTANT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 61 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_BIRTHDATE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 57 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_BUSINESS_EMAIL: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 34 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_BUSINESS_EMAIL2: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 35 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_BUSINESS_FAX: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 45 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_BUSINESS_FULL_POSTAL_ADDRESS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 17 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_BUSINESS_PHONE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 40 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_BUSINESS_PHONE2: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 41 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_BUSINESS_POSTAL_ADDRESS_CITY: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 20 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_BUSINESS_POSTAL_ADDRESS_COUNTRY: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 23 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_BUSINESS_POSTAL_ADDRESS_LINE1: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 18 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_BUSINESS_POSTAL_ADDRESS_LINE2: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 19 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_BUSINESS_POSTAL_ADDRESS_POSTAL_CODE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 22 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_BUSINESS_POSTAL_ADDRESS_REGION: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 21 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_BUSINESS_WEB_ADDRESS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 50 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_CHILDREN: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 60 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_COMPANY_NAME: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 54 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_DISPLAY_NAME: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_FIRST_NAME: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_INSTANT_MESSENGER: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 51 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_INSTANT_MESSENGER2: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 52 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_INSTANT_MESSENGER3: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 53 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_LAST_NAME: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 5 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_MIDDLE_NAMES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_MOBILE_PHONE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 42 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_MOBILE_PHONE2: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 43 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTACT_OBJECT_PROPERTIES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_OTHER_EMAILS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 36 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_OTHER_FULL_POSTAL_ADDRESS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 24 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_OTHER_PHONES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 47 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_OTHER_POSTAL_ADDRESS_CITY: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 27 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_OTHER_POSTAL_ADDRESS_LINE1: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 25 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_OTHER_POSTAL_ADDRESS_LINE2: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 26 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_OTHER_POSTAL_ADDRESS_POSTAL_CODE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 29 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_OTHER_POSTAL_ADDRESS_POSTAL_COUNTRY: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 30 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_OTHER_POSTAL_ADDRESS_REGION: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 28 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_PAGER: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 46 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_PERSONAL_EMAIL: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 32 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_PERSONAL_EMAIL2: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 33 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_PERSONAL_FAX: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 44 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_PERSONAL_FULL_POSTAL_ADDRESS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 10 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_PERSONAL_PHONE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 38 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_PERSONAL_PHONE2: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 39 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_PERSONAL_POSTAL_ADDRESS_CITY: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 13 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_PERSONAL_POSTAL_ADDRESS_COUNTRY: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 16 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_PERSONAL_POSTAL_ADDRESS_LINE1: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 11 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_PERSONAL_POSTAL_ADDRESS_LINE2: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 12 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_PERSONAL_POSTAL_ADDRESS_POSTAL_CODE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 15 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_PERSONAL_POSTAL_ADDRESS_REGION: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 14 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_PERSONAL_WEB_ADDRESS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 49 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_PHONETIC_COMPANY_NAME: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 55 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_PHONETIC_FIRST_NAME: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 8 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_PHONETIC_LAST_NAME: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 9 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_PREFIX: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 6 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_PRIMARY_EMAIL_ADDRESS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 31 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_PRIMARY_FAX: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 58 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_PRIMARY_PHONE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 37 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_PRIMARY_WEB_ADDRESS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 48 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_RINGTONE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 63 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_ROLE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 56 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_SPOUSE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 59 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_CONTACT_SUFFIX: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xfbd4fdab_987d_4777_b3f9_726185a9312b), pid: 7 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTENT_TYPE_ALL: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x80e170d2_1055_4a3e_b952_82cc4f8a8689);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTENT_TYPE_APPOINTMENT: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x0fed060e_8793_4b1e_90c9_48ac389ac631);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTENT_TYPE_AUDIO: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x4ad2c85e_5e2d_45e5_8864_4f229e3c6cf0);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTENT_TYPE_AUDIO_ALBUM: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xaa18737e_5009_48fa_ae21_85f24383b4e6);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTENT_TYPE_CALENDAR: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xa1fd5967_6023_49a0_9df1_f8060be751b0);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTENT_TYPE_CERTIFICATE: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xdc3876e8_a948_4060_9050_cbd77e8a3d87);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTENT_TYPE_CONTACT: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xeaba8313_4525_4707_9f0e_87c6808e9435);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTENT_TYPE_CONTACT_GROUP: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x346b8932_4c36_40d8_9415_1828291f9de9);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTENT_TYPE_DOCUMENT: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x680adf52_950a_4041_9b41_65e393648155);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTENT_TYPE_EMAIL: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x8038044a_7e51_4f8f_883d_1d0623d14533);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTENT_TYPE_FOLDER: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x27e2e392_a111_48e0_ab0c_e17705a05f85);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTENT_TYPE_FUNCTIONAL_OBJECT: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x99ed0160_17ff_4c44_9d98_1d7a6f941921);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTENT_TYPE_GENERIC_FILE: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x0085e0a6_8d34_45d7_bc5c_447e59c73d48);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTENT_TYPE_GENERIC_MESSAGE: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xe80eaaf8_b2db_4133_b67e_1bef4b4a6e5f);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTENT_TYPE_IMAGE: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xef2107d5_a52a_4243_a26b_62d4176d7603);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTENT_TYPE_IMAGE_ALBUM: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x75793148_15f5_4a30_a813_54ed8a37e226);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTENT_TYPE_MEDIA_CAST: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x5e88b3cc_3e65_4e62_bfff_229495253ab0);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTENT_TYPE_MEMO: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x9cd20ecf_3b50_414f_a641_e473ffe45751);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTENT_TYPE_MIXED_CONTENT_ALBUM: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x00f0c3ac_a593_49ac_9219_24abca5a2563);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTENT_TYPE_NETWORK_ASSOCIATION: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x031da7ee_18c8_4205_847e_89a11261d0f3);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTENT_TYPE_PLAYLIST: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x1a33f7e4_af13_48f5_994e_77369dfe04a3);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTENT_TYPE_PROGRAM: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xd269f96a_247c_4bff_98fb_97f3c49220e6);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTENT_TYPE_SECTION: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x821089f5_1d91_4dc9_be3c_bbb1b35b18ce);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTENT_TYPE_TASK: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x63252f2c_887f_4cb6_b1ac_d29855dcef6c);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTENT_TYPE_TELEVISION: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x60a169cf_f2ae_4e21_9375_9677f11c1c6e);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTENT_TYPE_UNSPECIFIED: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x28d8d31e_249c_454e_aabc_34883168e634);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTENT_TYPE_VIDEO: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x9261b03c_3d78_4519_85e3_02c5e1f50bb9);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTENT_TYPE_VIDEO_ALBUM: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x012b0db7_d4c1_45d6_b081_94b87779614f);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTENT_TYPE_WIRELESS_PROFILE: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x0bac070a_9f5f_4da4_a8f6_3de44d68fd6c);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CONTROL_FUNCTION_GENERIC_MESSAGE: u32 = 66u32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_DEVICE_DATETIME: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x26d4979a_e643_4626_9e2b_736dc0c92fdc), pid: 11 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_DEVICE_EDP_IDENTITY: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x6c2b878c_c2ec_490d_b425_d7a75e23e5ed), pid: 1 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_DEVICE_FIRMWARE_VERSION: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x26d4979a_e643_4626_9e2b_736dc0c92fdc), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_DEVICE_FRIENDLY_NAME: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x26d4979a_e643_4626_9e2b_736dc0c92fdc), pid: 12 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_DEVICE_FUNCTIONAL_UNIQUE_ID: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x463dd662_7fc4_4291_911c_7f4c9cca9799), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_DEVICE_MANUFACTURER: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x26d4979a_e643_4626_9e2b_736dc0c92fdc), pid: 7 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_DEVICE_MODEL: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x26d4979a_e643_4626_9e2b_736dc0c92fdc), pid: 8 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_DEVICE_MODEL_UNIQUE_ID: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x463dd662_7fc4_4291_911c_7f4c9cca9799), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_DEVICE_NETWORK_IDENTIFIER: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x26d4979a_e643_4626_9e2b_736dc0c92fdc), pid: 16 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_DEVICE_OBJECT_ID: ::windows_sys::core::PCWSTR = ::windows_sys::core::w!("DEVICE");
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_DEVICE_POWER_LEVEL: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x26d4979a_e643_4626_9e2b_736dc0c92fdc), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_DEVICE_POWER_SOURCE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x26d4979a_e643_4626_9e2b_736dc0c92fdc), pid: 5 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_DEVICE_PROPERTIES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x26d4979a_e643_4626_9e2b_736dc0c92fdc);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_DEVICE_PROPERTIES_V2: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x463dd662_7fc4_4291_911c_7f4c9cca9799);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_DEVICE_PROPERTIES_V3: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x6c2b878c_c2ec_490d_b425_d7a75e23e5ed);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_DEVICE_PROTOCOL: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x26d4979a_e643_4626_9e2b_736dc0c92fdc), pid: 6 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_DEVICE_SERIAL_NUMBER: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x26d4979a_e643_4626_9e2b_736dc0c92fdc), pid: 9 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_DEVICE_SUPPORTED_DRM_SCHEMES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x26d4979a_e643_4626_9e2b_736dc0c92fdc), pid: 13 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_DEVICE_SUPPORTED_FORMATS_ARE_ORDERED: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x26d4979a_e643_4626_9e2b_736dc0c92fdc), pid: 14 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_DEVICE_SUPPORTS_NON_CONSUMABLE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x26d4979a_e643_4626_9e2b_736dc0c92fdc), pid: 10 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_DEVICE_SYNC_PARTNER: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x26d4979a_e643_4626_9e2b_736dc0c92fdc), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_DEVICE_TRANSPORT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x463dd662_7fc4_4291_911c_7f4c9cca9799), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_DEVICE_TYPE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x26d4979a_e643_4626_9e2b_736dc0c92fdc), pid: 15 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_DEVICE_USE_DEVICE_STAGE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x463dd662_7fc4_4291_911c_7f4c9cca9799), pid: 5 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_DOCUMENT_OBJECT_PROPERTIES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x0b110203_eb95_4f02_93e0_97c631493ad5);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_EMAIL_BCC_LINE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x41f8f65a_5484_4782_b13d_4740dd7c37c5), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_EMAIL_CC_LINE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x41f8f65a_5484_4782_b13d_4740dd7c37c5), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_EMAIL_HAS_ATTACHMENTS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x41f8f65a_5484_4782_b13d_4740dd7c37c5), pid: 9 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_EMAIL_HAS_BEEN_READ: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x41f8f65a_5484_4782_b13d_4740dd7c37c5), pid: 7 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EMAIL_OBJECT_PROPERTIES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x41f8f65a_5484_4782_b13d_4740dd7c37c5);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_EMAIL_RECEIVED_TIME: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x41f8f65a_5484_4782_b13d_4740dd7c37c5), pid: 8 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_EMAIL_SENDER_ADDRESS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x41f8f65a_5484_4782_b13d_4740dd7c37c5), pid: 10 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_EMAIL_TO_LINE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x41f8f65a_5484_4782_b13d_4740dd7c37c5), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EVENT_ATTRIBUTES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x10c96578_2e81_4111_adde_e08ca6138f6d);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_EVENT_ATTRIBUTE_NAME: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x10c96578_2e81_4111_adde_e08ca6138f6d), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_EVENT_ATTRIBUTE_OPTIONS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x10c96578_2e81_4111_adde_e08ca6138f6d), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_EVENT_ATTRIBUTE_PARAMETERS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x10c96578_2e81_4111_adde_e08ca6138f6d), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EVENT_DEVICE_CAPABILITIES_UPDATED: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x36885aa1_cd54_4daa_b3d0_afb3e03f5999);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EVENT_DEVICE_REMOVED: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xe4cbca1b_6918_48b9_85ee_02be7c850af9);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EVENT_DEVICE_RESET: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x7755cf53_c1ed_44f3_b5a2_451e2c376b27);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EVENT_MTP_VENDOR_EXTENDED_EVENTS: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x00000000_5738_4ff2_8445_be3126691059);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EVENT_NOTIFICATION: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x2ba2e40a_6b4c_4295_bb43_26322b99aeb2);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EVENT_OBJECT_ADDED: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xa726da95_e207_4b02_8d44_bef2e86cbffc);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EVENT_OBJECT_REMOVED: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xbe82ab88_a52c_4823_96e5_d0272671fc38);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EVENT_OBJECT_TRANSFER_REQUESTED: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x8d16a0a1_f2c6_41da_8f19_5e53721adbf2);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EVENT_OBJECT_UPDATED: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x1445a759_2e01_485d_9f27_ff07dae697ab);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EVENT_OPTIONS_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xb3d8dad7_a361_4b83_8a48_5b02ce10713b);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_EVENT_OPTION_IS_AUTOPLAY_EVENT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3d8dad7_a361_4b83_8a48_5b02ce10713b), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_EVENT_OPTION_IS_BROADCAST_EVENT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3d8dad7_a361_4b83_8a48_5b02ce10713b), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_EVENT_PARAMETER_CHILD_HIERARCHY_CHANGED: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x15ab1953_f817_4fef_a921_5676e838f6e0), pid: 8 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_EVENT_PARAMETER_EVENT_ID: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x15ab1953_f817_4fef_a921_5676e838f6e0), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_EVENT_PARAMETER_OBJECT_CREATION_COOKIE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x15ab1953_f817_4fef_a921_5676e838f6e0), pid: 7 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_EVENT_PARAMETER_OBJECT_PARENT_PERSISTENT_UNIQUE_ID: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x15ab1953_f817_4fef_a921_5676e838f6e0), pid: 6 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_EVENT_PARAMETER_OPERATION_PROGRESS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x15ab1953_f817_4fef_a921_5676e838f6e0), pid: 5 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_EVENT_PARAMETER_OPERATION_STATE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x15ab1953_f817_4fef_a921_5676e838f6e0), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_EVENT_PARAMETER_PNP_DEVICE_ID: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x15ab1953_f817_4fef_a921_5676e838f6e0), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_EVENT_PARAMETER_SERVICE_METHOD_CONTEXT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x52807b8a_4914_4323_9b9a_74f654b2b846), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EVENT_PROPERTIES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x15ab1953_f817_4fef_a921_5676e838f6e0);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EVENT_PROPERTIES_V2: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x52807b8a_4914_4323_9b9a_74f654b2b846);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EVENT_SERVICE_METHOD_COMPLETE: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x8a33f5f8_0acc_4d9b_9cc4_112d353b86ca);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EVENT_STORAGE_FORMAT: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x3782616b_22bc_4474_a251_3070f8d38857);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_FOLDER_CONTENT_TYPES_ALLOWED: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x7e9a7abf_e568_4b34_aa2f_13bb12ab177d), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_FOLDER_OBJECT_PROPERTIES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x7e9a7abf_e568_4b34_aa2f_13bb12ab177d);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_FORMAT_ATTRIBUTES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xa0a02000_bcaf_4be8_b3f5_233f231cf58f);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_FORMAT_ATTRIBUTE_MIMETYPE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xa0a02000_bcaf_4be8_b3f5_233f231cf58f), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_FORMAT_ATTRIBUTE_NAME: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xa0a02000_bcaf_4be8_b3f5_233f231cf58f), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_FUNCTIONAL_CATEGORY_ALL: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x2d8a6512_a74c_448e_ba8a_f4ac07c49399);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_FUNCTIONAL_CATEGORY_AUDIO_CAPTURE: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x3f2a1919_c7c2_4a00_855d_f57cf06debbb);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_FUNCTIONAL_CATEGORY_DEVICE: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x08ea466b_e3a4_4336_a1f3_a44d2b5c438c);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_FUNCTIONAL_CATEGORY_NETWORK_CONFIGURATION: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x48f4db72_7c6a_4ab0_9e1a_470e3cdbf26a);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_FUNCTIONAL_CATEGORY_RENDERING_INFORMATION: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x08600ba4_a7ba_4a01_ab0e_0065d0a356d3);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_FUNCTIONAL_CATEGORY_SMS: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x0044a0b1_c1e9_4afd_b358_a62c6117c9cf);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_FUNCTIONAL_CATEGORY_STILL_IMAGE_CAPTURE: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x613ca327_ab93_4900_b4fa_895bb5874b79);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_FUNCTIONAL_CATEGORY_STORAGE: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x23f05bbc_15de_4c2a_a55b_a9af5ce412ef);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_FUNCTIONAL_CATEGORY_VIDEO_CAPTURE: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xe23e5f6b_7243_43aa_8df1_0eb3d968a918);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_FUNCTIONAL_OBJECT_CATEGORY: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x8f052d93_abca_4fc5_a5ac_b01df4dbe598), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_FUNCTIONAL_OBJECT_PROPERTIES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x8f052d93_abca_4fc5_a5ac_b01df4dbe598);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_IMAGE_BITDEPTH: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x63d64908_9fa1_479f_85ba_9952216447db), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_IMAGE_COLOR_CORRECTED_STATUS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x63d64908_9fa1_479f_85ba_9952216447db), pid: 5 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_IMAGE_CROPPED_STATUS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x63d64908_9fa1_479f_85ba_9952216447db), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_IMAGE_EXPOSURE_INDEX: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x63d64908_9fa1_479f_85ba_9952216447db), pid: 8 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_IMAGE_EXPOSURE_TIME: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x63d64908_9fa1_479f_85ba_9952216447db), pid: 7 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_IMAGE_FNUMBER: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x63d64908_9fa1_479f_85ba_9952216447db), pid: 6 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_IMAGE_HORIZONTAL_RESOLUTION: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x63d64908_9fa1_479f_85ba_9952216447db), pid: 9 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_IMAGE_OBJECT_PROPERTIES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x63d64908_9fa1_479f_85ba_9952216447db);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_IMAGE_VERTICAL_RESOLUTION: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x63d64908_9fa1_479f_85ba_9952216447db), pid: 10 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_ALBUM_ARTIST: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 25 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_ARTIST: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 24 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_AUDIO_ENCODING_PROFILE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 49 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_BITRATE_TYPE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_BUY_NOW: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 20 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_BYTE_BOOKMARK: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 36 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_COMPOSER: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 11 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_COPYRIGHT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_DESCRIPTION: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 31 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_DESTINATION_URL: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 30 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_DURATION: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 19 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_EFFECTIVE_RATING: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 12 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_ENCODING_PROFILE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 21 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_GENRE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 32 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_GUID: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 38 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_HEIGHT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 23 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_LAST_ACCESSED_TIME: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 8 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_LAST_BUILD_DATE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 35 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_MANAGING_EDITOR: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 27 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_META_GENRE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 10 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_OBJECT_BOOKMARK: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 34 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_OWNER: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 26 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_PARENTAL_RATING: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 9 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_MEDIA_PROPERTIES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_RELEASE_DATE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 14 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_SAMPLE_RATE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 15 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_SKIP_COUNT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 7 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_SOURCE_URL: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 29 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_STAR_RATING: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 16 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_SUBSCRIPTION_CONTENT_ID: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 5 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_SUB_DESCRIPTION: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 39 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_SUB_TITLE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 13 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_TIME_BOOKMARK: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 33 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_TIME_TO_LIVE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 37 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_TITLE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 18 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_TOTAL_BITRATE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_USER_EFFECTIVE_RATING: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 17 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_USE_COUNT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 6 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_WEBMASTER: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 28 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MEDIA_WIDTH: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2ed8ba05_0ad3_42dc_b0d0_bc95ac396ac8), pid: 22 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_MEMO_OBJECT_PROPERTIES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x5ffbfc7b_7483_41ad_afb9_da3f4e592b8d);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_METHOD_ATTRIBUTES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xf17a5071_f039_44af_8efe_432cf32e432a);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_METHOD_ATTRIBUTE_ACCESS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xf17a5071_f039_44af_8efe_432cf32e432a), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_METHOD_ATTRIBUTE_ASSOCIATED_FORMAT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xf17a5071_f039_44af_8efe_432cf32e432a), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_METHOD_ATTRIBUTE_NAME: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xf17a5071_f039_44af_8efe_432cf32e432a), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_METHOD_ATTRIBUTE_PARAMETERS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xf17a5071_f039_44af_8efe_432cf32e432a), pid: 5 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MUSIC_ALBUM: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb324f56a_dc5d_46e5_b6df_d2ea414888c6), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MUSIC_LYRICS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb324f56a_dc5d_46e5_b6df_d2ea414888c6), pid: 6 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MUSIC_MOOD: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb324f56a_dc5d_46e5_b6df_d2ea414888c6), pid: 8 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_MUSIC_OBJECT_PROPERTIES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xb324f56a_dc5d_46e5_b6df_d2ea414888c6);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_MUSIC_TRACK: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb324f56a_dc5d_46e5_b6df_d2ea414888c6), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_NETWORK_ASSOCIATION_HOST_NETWORK_IDENTIFIERS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xe4c93c1f_b203_43f1_a100_5a07d11b0274), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_NETWORK_ASSOCIATION_PROPERTIES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xe4c93c1f_b203_43f1_a100_5a07d11b0274);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_NETWORK_ASSOCIATION_X509V3SEQUENCE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xe4c93c1f_b203_43f1_a100_5a07d11b0274), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OBJECT_BACK_REFERENCES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef6b490d_5cd8_437a_affc_da8b60ee4a3c), pid: 21 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OBJECT_CAN_DELETE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef6b490d_5cd8_437a_affc_da8b60ee4a3c), pid: 26 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OBJECT_CONTAINER_FUNCTIONAL_OBJECT_ID: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef6b490d_5cd8_437a_affc_da8b60ee4a3c), pid: 23 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OBJECT_CONTENT_TYPE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef6b490d_5cd8_437a_affc_da8b60ee4a3c), pid: 7 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OBJECT_DATE_AUTHORED: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef6b490d_5cd8_437a_affc_da8b60ee4a3c), pid: 20 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OBJECT_DATE_CREATED: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef6b490d_5cd8_437a_affc_da8b60ee4a3c), pid: 18 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OBJECT_DATE_MODIFIED: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef6b490d_5cd8_437a_affc_da8b60ee4a3c), pid: 19 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OBJECT_FORMAT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef6b490d_5cd8_437a_affc_da8b60ee4a3c), pid: 6 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_3G2: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xb9850000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_3G2A: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x1a11202d_8759_4e34_ba5e_b1211087eee4);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_3GP: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xb9840000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_3GPA: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xe5172730_f971_41ef_a10b_2271a0019d7a);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_AAC: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xb9030000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_ABSTRACT_CONTACT: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xbb810000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_ABSTRACT_CONTACT_GROUP: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xba060000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_ABSTRACT_MEDIA_CAST: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xba0b0000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_AIFF: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x30070000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_ALL: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xc1f62eb2_4bb3_479c_9cfa_05b5f3a57b22);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_AMR: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xb9080000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_ASF: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x300c0000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_ASXPLAYLIST: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xba130000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_ATSCTS: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xb9870000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_AUDIBLE: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xb9040000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_AVCHD: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xb9860000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_AVI: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x300a0000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_BMP: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x38040000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_CIFF: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x38050000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_DPOF: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x30060000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_DVBTS: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xb9880000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_EXECUTABLE: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x30030000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_EXIF: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x38010000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_FLAC: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xb9060000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_FLASHPIX: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x38030000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_GIF: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x38070000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_HTML: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x30050000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_ICALENDAR: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xbe030000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_ICON: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x077232ed_102c_4638_9c22_83f142bfc822);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_JFIF: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x38080000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_JP2: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x380f0000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_JPEGXR: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xb8040000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_JPX: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x38100000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_M3UPLAYLIST: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xba110000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_M4A: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x30aba7ac_6ffd_4c23_a359_3e9b52f3f1c8);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_MHT_COMPILED_HTML: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xba840000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_MICROSOFT_EXCEL: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xba850000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_MICROSOFT_POWERPOINT: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xba860000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_MICROSOFT_WFC: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xb1040000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_MICROSOFT_WORD: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xba830000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_MKV: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xb9900000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_MP2: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xb9830000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_MP3: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x30090000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_MP4: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xb9820000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_MPEG: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x300b0000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_MPLPLAYLIST: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xba120000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_NETWORK_ASSOCIATION: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xb1020000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_OGG: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xb9020000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_PCD: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x38090000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_PICT: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x380a0000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_PLSPLAYLIST: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xba140000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_PNG: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x380b0000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_PROPERTIES_ONLY: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x30010000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_QCELP: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xb9070000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_SCRIPT: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x30020000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_TEXT: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x30040000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_TIFF: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x380d0000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_TIFFEP: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x38020000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_TIFFIT: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x380e0000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_UNSPECIFIED: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x30000000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_VCALENDAR1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xbe020000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_VCARD2: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xbb820000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_VCARD3: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xbb830000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_WAVE: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x30080000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_WBMP: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xb8030000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_WINDOWSIMAGEFORMAT: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xb8810000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_WMA: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xb9010000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_WMV: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xb9810000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_WPLPLAYLIST: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xba100000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_X509V3CERTIFICATE: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xb1030000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_FORMAT_XML: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xba820000_ae6c_4804_98ba_c57b46965fe7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OBJECT_GENERATE_THUMBNAIL_FROM_RESOURCE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef6b490d_5cd8_437a_affc_da8b60ee4a3c), pid: 24 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OBJECT_HINT_LOCATION_DISPLAY_NAME: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef6b490d_5cd8_437a_affc_da8b60ee4a3c), pid: 25 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OBJECT_ID: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef6b490d_5cd8_437a_affc_da8b60ee4a3c), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OBJECT_ISHIDDEN: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef6b490d_5cd8_437a_affc_da8b60ee4a3c), pid: 9 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OBJECT_ISSYSTEM: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef6b490d_5cd8_437a_affc_da8b60ee4a3c), pid: 10 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OBJECT_IS_DRM_PROTECTED: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef6b490d_5cd8_437a_affc_da8b60ee4a3c), pid: 17 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OBJECT_KEYWORDS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef6b490d_5cd8_437a_affc_da8b60ee4a3c), pid: 15 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OBJECT_LANGUAGE_LOCALE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef6b490d_5cd8_437a_affc_da8b60ee4a3c), pid: 27 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OBJECT_NAME: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef6b490d_5cd8_437a_affc_da8b60ee4a3c), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OBJECT_NON_CONSUMABLE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef6b490d_5cd8_437a_affc_da8b60ee4a3c), pid: 13 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OBJECT_ORIGINAL_FILE_NAME: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef6b490d_5cd8_437a_affc_da8b60ee4a3c), pid: 12 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OBJECT_PARENT_ID: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef6b490d_5cd8_437a_affc_da8b60ee4a3c), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OBJECT_PERSISTENT_UNIQUE_ID: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef6b490d_5cd8_437a_affc_da8b60ee4a3c), pid: 5 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_PROPERTIES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xef6b490d_5cd8_437a_affc_da8b60ee4a3c);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OBJECT_PROPERTIES_V2: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x0373cd3d_4a46_40d7_b4d8_73e8da74e775);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OBJECT_REFERENCES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef6b490d_5cd8_437a_affc_da8b60ee4a3c), pid: 14 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OBJECT_SIZE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef6b490d_5cd8_437a_affc_da8b60ee4a3c), pid: 11 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OBJECT_SUPPORTED_UNITS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x0373cd3d_4a46_40d7_b4d8_73e8da74e775), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OBJECT_SYNC_ID: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef6b490d_5cd8_437a_affc_da8b60ee4a3c), pid: 16 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OPTION_OBJECT_MANAGEMENT_RECURSIVE_DELETE_SUPPORTED: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef1e43dd_a9ed_4341_8bcc_186192aea089), pid: 5001 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OPTION_OBJECT_RESOURCES_NO_INPUT_BUFFER_ON_READ: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 5003 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OPTION_OBJECT_RESOURCES_SEEK_ON_READ_SUPPORTED: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 5001 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OPTION_OBJECT_RESOURCES_SEEK_ON_WRITE_SUPPORTED: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 5002 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OPTION_SMS_BINARY_MESSAGE_SUPPORTED: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xafc25d66_fe0d_4114_9097_970c93e920d1), pid: 5001 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_OPTION_VALID_OBJECT_IDS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xf0422a9c_5dc8_4440_b5bd_5df28835658a), pid: 5001 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_PARAMETER_ATTRIBUTES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xe6864dd7_f325_45ea_a1d5_97cf73b6ca58);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PARAMETER_ATTRIBUTE_DEFAULT_VALUE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xe6864dd7_f325_45ea_a1d5_97cf73b6ca58), pid: 5 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PARAMETER_ATTRIBUTE_ENUMERATION_ELEMENTS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xe6864dd7_f325_45ea_a1d5_97cf73b6ca58), pid: 9 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PARAMETER_ATTRIBUTE_FORM: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xe6864dd7_f325_45ea_a1d5_97cf73b6ca58), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PARAMETER_ATTRIBUTE_MAX_SIZE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xe6864dd7_f325_45ea_a1d5_97cf73b6ca58), pid: 11 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PARAMETER_ATTRIBUTE_NAME: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xe6864dd7_f325_45ea_a1d5_97cf73b6ca58), pid: 13 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PARAMETER_ATTRIBUTE_ORDER: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xe6864dd7_f325_45ea_a1d5_97cf73b6ca58), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PARAMETER_ATTRIBUTE_RANGE_MAX: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xe6864dd7_f325_45ea_a1d5_97cf73b6ca58), pid: 7 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PARAMETER_ATTRIBUTE_RANGE_MIN: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xe6864dd7_f325_45ea_a1d5_97cf73b6ca58), pid: 6 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PARAMETER_ATTRIBUTE_RANGE_STEP: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xe6864dd7_f325_45ea_a1d5_97cf73b6ca58), pid: 8 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PARAMETER_ATTRIBUTE_REGULAR_EXPRESSION: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xe6864dd7_f325_45ea_a1d5_97cf73b6ca58), pid: 10 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PARAMETER_ATTRIBUTE_USAGE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xe6864dd7_f325_45ea_a1d5_97cf73b6ca58), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PARAMETER_ATTRIBUTE_VARTYPE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xe6864dd7_f325_45ea_a1d5_97cf73b6ca58), pid: 12 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_PROPERTIES_MTP_VENDOR_EXTENDED_DEVICE_PROPS: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x4d545058_8900_40b3_8f1d_dc246e1e8370);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_PROPERTIES_MTP_VENDOR_EXTENDED_OBJECT_PROPS: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x4d545058_4fce_4578_95c8_8698a9bc0f49);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_PROPERTY_ATTRIBUTES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xab7943d8_6332_445f_a00d_8d5ef1e96f37);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_PROPERTY_ATTRIBUTES_V2: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x5d9da160_74ae_43cc_85a9_fe555a80798e);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_ATTRIBUTE_CAN_DELETE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xab7943d8_6332_445f_a00d_8d5ef1e96f37), pid: 5 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_ATTRIBUTE_CAN_READ: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xab7943d8_6332_445f_a00d_8d5ef1e96f37), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_ATTRIBUTE_CAN_WRITE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xab7943d8_6332_445f_a00d_8d5ef1e96f37), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_ATTRIBUTE_DEFAULT_VALUE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xab7943d8_6332_445f_a00d_8d5ef1e96f37), pid: 6 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_ATTRIBUTE_ENUMERATION_ELEMENTS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xab7943d8_6332_445f_a00d_8d5ef1e96f37), pid: 11 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_ATTRIBUTE_FAST_PROPERTY: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xab7943d8_6332_445f_a00d_8d5ef1e96f37), pid: 7 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_ATTRIBUTE_FORM: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xab7943d8_6332_445f_a00d_8d5ef1e96f37), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_ATTRIBUTE_MAX_SIZE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xab7943d8_6332_445f_a00d_8d5ef1e96f37), pid: 13 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_ATTRIBUTE_NAME: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x5d9da160_74ae_43cc_85a9_fe555a80798e), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_ATTRIBUTE_RANGE_MAX: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xab7943d8_6332_445f_a00d_8d5ef1e96f37), pid: 9 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_ATTRIBUTE_RANGE_MIN: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xab7943d8_6332_445f_a00d_8d5ef1e96f37), pid: 8 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_ATTRIBUTE_RANGE_STEP: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xab7943d8_6332_445f_a00d_8d5ef1e96f37), pid: 10 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_ATTRIBUTE_REGULAR_EXPRESSION: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xab7943d8_6332_445f_a00d_8d5ef1e96f37), pid: 12 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_ATTRIBUTE_VARTYPE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x5d9da160_74ae_43cc_85a9_fe555a80798e), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_CAPABILITIES_COMMAND: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x0cabec78_6b74_41c6_9216_2639d1fce356), pid: 1002 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_CAPABILITIES_COMMAND_OPTIONS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x0cabec78_6b74_41c6_9216_2639d1fce356), pid: 1003 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_CAPABILITIES_CONTENT_TYPE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x0cabec78_6b74_41c6_9216_2639d1fce356), pid: 1008 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_CAPABILITIES_CONTENT_TYPES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x0cabec78_6b74_41c6_9216_2639d1fce356), pid: 1007 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_CAPABILITIES_EVENT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x0cabec78_6b74_41c6_9216_2639d1fce356), pid: 1014 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_CAPABILITIES_EVENT_OPTIONS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x0cabec78_6b74_41c6_9216_2639d1fce356), pid: 1015 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_CAPABILITIES_FORMAT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x0cabec78_6b74_41c6_9216_2639d1fce356), pid: 1010 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_CAPABILITIES_FORMATS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x0cabec78_6b74_41c6_9216_2639d1fce356), pid: 1009 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_CAPABILITIES_FUNCTIONAL_CATEGORIES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x0cabec78_6b74_41c6_9216_2639d1fce356), pid: 1004 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_CAPABILITIES_FUNCTIONAL_CATEGORY: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x0cabec78_6b74_41c6_9216_2639d1fce356), pid: 1005 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_CAPABILITIES_FUNCTIONAL_OBJECTS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x0cabec78_6b74_41c6_9216_2639d1fce356), pid: 1006 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_CAPABILITIES_PROPERTY_ATTRIBUTES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x0cabec78_6b74_41c6_9216_2639d1fce356), pid: 1012 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_CAPABILITIES_PROPERTY_KEYS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x0cabec78_6b74_41c6_9216_2639d1fce356), pid: 1011 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_CAPABILITIES_SUPPORTED_COMMANDS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x0cabec78_6b74_41c6_9216_2639d1fce356), pid: 1001 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_CAPABILITIES_SUPPORTED_EVENTS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x0cabec78_6b74_41c6_9216_2639d1fce356), pid: 1013 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_CLASS_EXTENSION_DEVICE_INFORMATION_VALUES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x33fb0d11_64a3_4fac_b4c7_3dfeaa99b051), pid: 1001 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_CLASS_EXTENSION_DEVICE_INFORMATION_WRITE_RESULTS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x33fb0d11_64a3_4fac_b4c7_3dfeaa99b051), pid: 1002 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_CLASS_EXTENSION_SERVICE_INTERFACES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x7f0779b5_fa2b_4766_9cb2_f73ba30b6758), pid: 1002 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_CLASS_EXTENSION_SERVICE_OBJECT_ID: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x7f0779b5_fa2b_4766_9cb2_f73ba30b6758), pid: 1001 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_CLASS_EXTENSION_SERVICE_REGISTRATION_RESULTS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x7f0779b5_fa2b_4766_9cb2_f73ba30b6758), pid: 1003 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_COMMON_ACTIVITY_ID: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xf0422a9c_5dc8_4440_b5bd_5df28835658a), pid: 1011 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_COMMON_CLIENT_INFORMATION: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xf0422a9c_5dc8_4440_b5bd_5df28835658a), pid: 1009 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_COMMON_CLIENT_INFORMATION_CONTEXT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xf0422a9c_5dc8_4440_b5bd_5df28835658a), pid: 1010 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_COMMON_COMMAND_CATEGORY: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xf0422a9c_5dc8_4440_b5bd_5df28835658a), pid: 1001 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_COMMON_COMMAND_ID: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xf0422a9c_5dc8_4440_b5bd_5df28835658a), pid: 1002 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_COMMON_COMMAND_TARGET: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xf0422a9c_5dc8_4440_b5bd_5df28835658a), pid: 1006 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_COMMON_DRIVER_ERROR_CODE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xf0422a9c_5dc8_4440_b5bd_5df28835658a), pid: 1004 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_COMMON_HRESULT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xf0422a9c_5dc8_4440_b5bd_5df28835658a), pid: 1003 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_COMMON_OBJECT_IDS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xf0422a9c_5dc8_4440_b5bd_5df28835658a), pid: 1008 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_COMMON_PERSISTENT_UNIQUE_IDS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xf0422a9c_5dc8_4440_b5bd_5df28835658a), pid: 1007 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_DEVICE_HINTS_CONTENT_LOCATIONS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x0d5fb92b_cb46_4c4f_8343_0bc3d3f17c84), pid: 1002 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_DEVICE_HINTS_CONTENT_TYPE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x0d5fb92b_cb46_4c4f_8343_0bc3d3f17c84), pid: 1001 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_MTP_EXT_EVENT_PARAMS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x4d545058_ef88_4e4d_95c3_4f327f728a96), pid: 1011 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_MTP_EXT_OPERATION_CODE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x4d545058_1a2e_4106_a357_771e0819fc56), pid: 1001 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_MTP_EXT_OPERATION_PARAMS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x4d545058_1a2e_4106_a357_771e0819fc56), pid: 1002 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_MTP_EXT_OPTIMAL_TRANSFER_BUFFER_SIZE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x4d545058_1a2e_4106_a357_771e0819fc56), pid: 1013 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_MTP_EXT_RESPONSE_CODE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x4d545058_1a2e_4106_a357_771e0819fc56), pid: 1003 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_MTP_EXT_RESPONSE_PARAMS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x4d545058_1a2e_4106_a357_771e0819fc56), pid: 1004 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_MTP_EXT_TRANSFER_CONTEXT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x4d545058_1a2e_4106_a357_771e0819fc56), pid: 1006 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_MTP_EXT_TRANSFER_DATA: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x4d545058_1a2e_4106_a357_771e0819fc56), pid: 1012 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_MTP_EXT_TRANSFER_NUM_BYTES_READ: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x4d545058_1a2e_4106_a357_771e0819fc56), pid: 1009 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_MTP_EXT_TRANSFER_NUM_BYTES_TO_READ: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x4d545058_1a2e_4106_a357_771e0819fc56), pid: 1008 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_MTP_EXT_TRANSFER_NUM_BYTES_TO_WRITE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x4d545058_1a2e_4106_a357_771e0819fc56), pid: 1010 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_MTP_EXT_TRANSFER_NUM_BYTES_WRITTEN: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x4d545058_1a2e_4106_a357_771e0819fc56), pid: 1011 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_MTP_EXT_TRANSFER_TOTAL_DATA_SIZE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x4d545058_1a2e_4106_a357_771e0819fc56), pid: 1007 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_MTP_EXT_VENDOR_EXTENSION_DESCRIPTION: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x4d545058_1a2e_4106_a357_771e0819fc56), pid: 1014 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_MTP_EXT_VENDOR_OPERATION_CODES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x4d545058_1a2e_4106_a357_771e0819fc56), pid: 1005 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_NULL: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x00000000_0000_0000_0000_000000000000), pid: 0 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_ENUMERATION_CONTEXT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb7474e91_e7f8_4ad9_b400_ad1a4b58eeec), pid: 1004 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_ENUMERATION_FILTER: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb7474e91_e7f8_4ad9_b400_ad1a4b58eeec), pid: 1002 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_ENUMERATION_NUM_OBJECTS_REQUESTED: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb7474e91_e7f8_4ad9_b400_ad1a4b58eeec), pid: 1005 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_ENUMERATION_OBJECT_IDS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb7474e91_e7f8_4ad9_b400_ad1a4b58eeec), pid: 1003 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_ENUMERATION_PARENT_ID: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb7474e91_e7f8_4ad9_b400_ad1a4b58eeec), pid: 1001 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_MANAGEMENT_CONTEXT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef1e43dd_a9ed_4341_8bcc_186192aea089), pid: 1002 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_MANAGEMENT_COPY_RESULTS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef1e43dd_a9ed_4341_8bcc_186192aea089), pid: 1013 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_MANAGEMENT_CREATION_PROPERTIES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef1e43dd_a9ed_4341_8bcc_186192aea089), pid: 1001 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_MANAGEMENT_DATA: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef1e43dd_a9ed_4341_8bcc_186192aea089), pid: 1005 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_MANAGEMENT_DELETE_OPTIONS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef1e43dd_a9ed_4341_8bcc_186192aea089), pid: 1007 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_MANAGEMENT_DELETE_RESULTS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef1e43dd_a9ed_4341_8bcc_186192aea089), pid: 1010 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_MANAGEMENT_DESTINATION_FOLDER_OBJECT_ID: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef1e43dd_a9ed_4341_8bcc_186192aea089), pid: 1011 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_MANAGEMENT_MOVE_RESULTS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef1e43dd_a9ed_4341_8bcc_186192aea089), pid: 1012 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_MANAGEMENT_NUM_BYTES_TO_WRITE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef1e43dd_a9ed_4341_8bcc_186192aea089), pid: 1003 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_MANAGEMENT_NUM_BYTES_WRITTEN: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef1e43dd_a9ed_4341_8bcc_186192aea089), pid: 1004 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_MANAGEMENT_OBJECT_FORMAT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef1e43dd_a9ed_4341_8bcc_186192aea089), pid: 1016 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_MANAGEMENT_OBJECT_ID: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef1e43dd_a9ed_4341_8bcc_186192aea089), pid: 1006 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_MANAGEMENT_OBJECT_IDS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef1e43dd_a9ed_4341_8bcc_186192aea089), pid: 1009 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_MANAGEMENT_OPTIMAL_TRANSFER_BUFFER_SIZE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef1e43dd_a9ed_4341_8bcc_186192aea089), pid: 1008 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_MANAGEMENT_PROPERTY_KEYS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef1e43dd_a9ed_4341_8bcc_186192aea089), pid: 1015 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_MANAGEMENT_UPDATE_PROPERTIES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xef1e43dd_a9ed_4341_8bcc_186192aea089), pid: 1014 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_PROPERTIES_BULK_CONTEXT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x11c824dd_04cd_4e4e_8c7b_f6efb794d84e), pid: 1002 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_PROPERTIES_BULK_DEPTH: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x11c824dd_04cd_4e4e_8c7b_f6efb794d84e), pid: 1005 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_PROPERTIES_BULK_OBJECT_FORMAT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x11c824dd_04cd_4e4e_8c7b_f6efb794d84e), pid: 1007 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_PROPERTIES_BULK_OBJECT_IDS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x11c824dd_04cd_4e4e_8c7b_f6efb794d84e), pid: 1001 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_PROPERTIES_BULK_PARENT_OBJECT_ID: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x11c824dd_04cd_4e4e_8c7b_f6efb794d84e), pid: 1006 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_PROPERTIES_BULK_PROPERTY_KEYS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x11c824dd_04cd_4e4e_8c7b_f6efb794d84e), pid: 1004 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_PROPERTIES_BULK_VALUES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x11c824dd_04cd_4e4e_8c7b_f6efb794d84e), pid: 1003 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_PROPERTIES_BULK_WRITE_RESULTS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x11c824dd_04cd_4e4e_8c7b_f6efb794d84e), pid: 1008 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_PROPERTIES_OBJECT_ID: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x9e5582e4_0814_44e6_981a_b2998d583804), pid: 1001 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_PROPERTIES_PROPERTY_ATTRIBUTES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x9e5582e4_0814_44e6_981a_b2998d583804), pid: 1003 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_PROPERTIES_PROPERTY_DELETE_RESULTS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x9e5582e4_0814_44e6_981a_b2998d583804), pid: 1006 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_PROPERTIES_PROPERTY_KEYS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x9e5582e4_0814_44e6_981a_b2998d583804), pid: 1002 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_PROPERTIES_PROPERTY_VALUES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x9e5582e4_0814_44e6_981a_b2998d583804), pid: 1004 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_PROPERTIES_PROPERTY_WRITE_RESULTS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x9e5582e4_0814_44e6_981a_b2998d583804), pid: 1005 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_RESOURCES_ACCESS_MODE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 1002 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_RESOURCES_CONTEXT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 1005 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_RESOURCES_DATA: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 1010 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_RESOURCES_NUM_BYTES_READ: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 1007 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_RESOURCES_NUM_BYTES_TO_READ: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 1006 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_RESOURCES_NUM_BYTES_TO_WRITE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 1008 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_RESOURCES_NUM_BYTES_WRITTEN: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 1009 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_RESOURCES_OBJECT_ID: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 1001 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_RESOURCES_OPTIMAL_TRANSFER_BUFFER_SIZE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 1011 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_RESOURCES_POSITION_FROM_START: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 1014 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_RESOURCES_RESOURCE_ATTRIBUTES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 1004 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_RESOURCES_RESOURCE_KEYS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 1003 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_RESOURCES_SEEK_OFFSET: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 1012 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_RESOURCES_SEEK_ORIGIN_FLAG: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 1013 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_RESOURCES_STREAM_UNITS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 1016 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_OBJECT_RESOURCES_SUPPORTS_UNITS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb3a2b22d_a595_4108_be0a_fc3c965f3d4a), pid: 1015 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_PUBLIC_KEY: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x78f9c6fc_79b8_473c_9060_6bd23dd072c4), pid: 1001 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_SERVICE_CAPABILITIES_COMMAND: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 1018 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_SERVICE_CAPABILITIES_COMMAND_OPTIONS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 1019 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_SERVICE_CAPABILITIES_EVENT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 1012 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_SERVICE_CAPABILITIES_EVENT_ATTRIBUTES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 1013 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_SERVICE_CAPABILITIES_FORMAT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 1002 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_SERVICE_CAPABILITIES_FORMATS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 1007 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_SERVICE_CAPABILITIES_FORMAT_ATTRIBUTES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 1008 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_SERVICE_CAPABILITIES_INHERITANCE_TYPE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 1014 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_SERVICE_CAPABILITIES_INHERITED_SERVICES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 1015 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_SERVICE_CAPABILITIES_METHOD: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 1003 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_SERVICE_CAPABILITIES_METHOD_ATTRIBUTES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 1004 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_SERVICE_CAPABILITIES_PARAMETER: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 1005 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_SERVICE_CAPABILITIES_PARAMETER_ATTRIBUTES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 1006 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_SERVICE_CAPABILITIES_PROPERTY_ATTRIBUTES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 1010 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_SERVICE_CAPABILITIES_PROPERTY_KEYS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 1009 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_SERVICE_CAPABILITIES_RENDERING_PROFILES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 1016 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_SERVICE_CAPABILITIES_SUPPORTED_COMMANDS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 1017 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_SERVICE_CAPABILITIES_SUPPORTED_EVENTS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 1011 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_SERVICE_CAPABILITIES_SUPPORTED_METHODS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x24457e74_2e9f_44f9_8c57_1d1bcb170b89), pid: 1001 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_SERVICE_METHOD: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2d521ca8_c1b0_4268_a342_cf19321569bc), pid: 1001 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_SERVICE_METHOD_CONTEXT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2d521ca8_c1b0_4268_a342_cf19321569bc), pid: 1004 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_SERVICE_METHOD_HRESULT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2d521ca8_c1b0_4268_a342_cf19321569bc), pid: 1005 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_SERVICE_METHOD_PARAMETER_VALUES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2d521ca8_c1b0_4268_a342_cf19321569bc), pid: 1002 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_SERVICE_METHOD_RESULT_VALUES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2d521ca8_c1b0_4268_a342_cf19321569bc), pid: 1003 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_SERVICE_OBJECT_ID: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x322f071d_36ef_477f_b4b5_6f52d734baee), pid: 1001 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_SMS_BINARY_MESSAGE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xafc25d66_fe0d_4114_9097_970c93e920d1), pid: 1004 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_SMS_MESSAGE_TYPE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xafc25d66_fe0d_4114_9097_970c93e920d1), pid: 1002 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_SMS_RECIPIENT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xafc25d66_fe0d_4114_9097_970c93e920d1), pid: 1001 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_SMS_TEXT_MESSAGE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xafc25d66_fe0d_4114_9097_970c93e920d1), pid: 1003 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_STORAGE_DESTINATION_OBJECT_ID: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xd8f907a6_34cc_45fa_97fb_d007fa47ec94), pid: 1002 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_PROPERTY_STORAGE_OBJECT_ID: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xd8f907a6_34cc_45fa_97fb_d007fa47ec94), pid: 1001 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_RENDERING_INFORMATION_OBJECT_PROPERTIES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xc53d039f_ee23_4a31_8590_7639879870b4);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_RENDERING_INFORMATION_PROFILES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xc53d039f_ee23_4a31_8590_7639879870b4), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_RENDERING_INFORMATION_PROFILE_ENTRY_CREATABLE_RESOURCES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xc53d039f_ee23_4a31_8590_7639879870b4), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_RENDERING_INFORMATION_PROFILE_ENTRY_TYPE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xc53d039f_ee23_4a31_8590_7639879870b4), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_RESOURCE_ALBUM_ART: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xf02aa354_2300_4e2d_a1b9_3b6730f7fa21), pid: 0 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_RESOURCE_ATTRIBUTES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x1eb6f604_9278_429f_93cc_5bb8c06656b6);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_RESOURCE_ATTRIBUTE_CAN_DELETE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x1eb6f604_9278_429f_93cc_5bb8c06656b6), pid: 5 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_RESOURCE_ATTRIBUTE_CAN_READ: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x1eb6f604_9278_429f_93cc_5bb8c06656b6), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_RESOURCE_ATTRIBUTE_CAN_WRITE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x1eb6f604_9278_429f_93cc_5bb8c06656b6), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_RESOURCE_ATTRIBUTE_FORMAT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x1eb6f604_9278_429f_93cc_5bb8c06656b6), pid: 8 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_RESOURCE_ATTRIBUTE_OPTIMAL_READ_BUFFER_SIZE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x1eb6f604_9278_429f_93cc_5bb8c06656b6), pid: 6 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_RESOURCE_ATTRIBUTE_OPTIMAL_WRITE_BUFFER_SIZE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x1eb6f604_9278_429f_93cc_5bb8c06656b6), pid: 7 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_RESOURCE_ATTRIBUTE_RESOURCE_KEY: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x1eb6f604_9278_429f_93cc_5bb8c06656b6), pid: 9 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_RESOURCE_ATTRIBUTE_TOTAL_SIZE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x1eb6f604_9278_429f_93cc_5bb8c06656b6), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_RESOURCE_AUDIO_CLIP: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x3bc13982_85b1_48e0_95a6_8d3ad06be117), pid: 0 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_RESOURCE_BRANDING_ART: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb633b1ae_6caf_4a87_9589_22ded6dd5899), pid: 0 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_RESOURCE_CONTACT_PHOTO: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x2c4d6803_80ea_4580_af9a_5be1a23eddcb), pid: 0 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_RESOURCE_DEFAULT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xe81e79be_34f0_41bf_b53f_f1a06ae87842), pid: 0 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_RESOURCE_GENERIC: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb9b9f515_ba70_4647_94dc_fa4925e95a07), pid: 0 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_RESOURCE_ICON: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xf195fed8_aa28_4ee3_b153_e182dd5edc39), pid: 0 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_RESOURCE_THUMBNAIL: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xc7c407ba_98fa_46b5_9960_23fec124cfde), pid: 0 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_RESOURCE_VIDEO_CLIP: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xb566ee42_6368_4290_8662_70182fb79f20), pid: 0 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_SECTION_DATA_LENGTH: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x516afd2b_c64e_44f0_98dc_bee1c88f7d66), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_SECTION_DATA_OFFSET: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x516afd2b_c64e_44f0_98dc_bee1c88f7d66), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_SECTION_DATA_REFERENCED_OBJECT_RESOURCE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x516afd2b_c64e_44f0_98dc_bee1c88f7d66), pid: 5 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_SECTION_DATA_UNITS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x516afd2b_c64e_44f0_98dc_bee1c88f7d66), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_SECTION_OBJECT_PROPERTIES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x516afd2b_c64e_44f0_98dc_bee1c88f7d66);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_SERVICE_PROPERTIES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x7510698a_cb54_481c_b8db_0d75c93f1c06);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_SERVICE_VERSION: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x7510698a_cb54_481c_b8db_0d75c93f1c06), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_SMS_ENCODING: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x7e1074cc_50ff_4dd1_a742_53be6f093a0d), pid: 5 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_SMS_MAX_PAYLOAD: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x7e1074cc_50ff_4dd1_a742_53be6f093a0d), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_SMS_OBJECT_PROPERTIES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x7e1074cc_50ff_4dd1_a742_53be6f093a0d);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_SMS_PROVIDER: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x7e1074cc_50ff_4dd1_a742_53be6f093a0d), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_SMS_TIMEOUT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x7e1074cc_50ff_4dd1_a742_53be6f093a0d), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STILL_IMAGE_ARTIST: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260), pid: 29 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STILL_IMAGE_BURST_INTERVAL: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260), pid: 24 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STILL_IMAGE_BURST_NUMBER: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260), pid: 23 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STILL_IMAGE_CAMERA_MANUFACTURER: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260), pid: 31 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STILL_IMAGE_CAMERA_MODEL: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260), pid: 30 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STILL_IMAGE_CAPTURE_DELAY: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260), pid: 17 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STILL_IMAGE_CAPTURE_FORMAT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STILL_IMAGE_CAPTURE_MODE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260), pid: 18 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_STILL_IMAGE_CAPTURE_OBJECT_PROPERTIES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STILL_IMAGE_CAPTURE_RESOLUTION: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STILL_IMAGE_COMPRESSION_SETTING: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STILL_IMAGE_CONTRAST: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260), pid: 19 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STILL_IMAGE_DIGITAL_ZOOM: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260), pid: 21 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STILL_IMAGE_EFFECT_MODE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260), pid: 22 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STILL_IMAGE_EXPOSURE_BIAS_COMPENSATION: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260), pid: 16 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STILL_IMAGE_EXPOSURE_INDEX: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260), pid: 15 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STILL_IMAGE_EXPOSURE_METERING_MODE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260), pid: 11 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STILL_IMAGE_EXPOSURE_PROGRAM_MODE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260), pid: 14 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STILL_IMAGE_EXPOSURE_TIME: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260), pid: 13 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STILL_IMAGE_FLASH_MODE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260), pid: 12 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STILL_IMAGE_FNUMBER: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260), pid: 7 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STILL_IMAGE_FOCAL_LENGTH: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260), pid: 8 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STILL_IMAGE_FOCUS_DISTANCE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260), pid: 9 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STILL_IMAGE_FOCUS_METERING_MODE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260), pid: 27 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STILL_IMAGE_FOCUS_MODE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260), pid: 10 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STILL_IMAGE_RGB_GAIN: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260), pid: 6 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STILL_IMAGE_SHARPNESS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260), pid: 20 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STILL_IMAGE_TIMELAPSE_INTERVAL: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260), pid: 26 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STILL_IMAGE_TIMELAPSE_NUMBER: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260), pid: 25 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STILL_IMAGE_UPLOAD_URL: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260), pid: 28 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STILL_IMAGE_WHITE_BALANCE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x58c571ec_1bcb_42a7_8ac5_bb291573a260), pid: 5 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STORAGE_ACCESS_CAPABILITY: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x01a3057a_74d6_4e80_bea7_dc4c212ce50a), pid: 11 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STORAGE_CAPACITY: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x01a3057a_74d6_4e80_bea7_dc4c212ce50a), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STORAGE_CAPACITY_IN_OBJECTS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x01a3057a_74d6_4e80_bea7_dc4c212ce50a), pid: 10 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STORAGE_DESCRIPTION: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x01a3057a_74d6_4e80_bea7_dc4c212ce50a), pid: 7 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STORAGE_FILE_SYSTEM_TYPE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x01a3057a_74d6_4e80_bea7_dc4c212ce50a), pid: 3 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STORAGE_FREE_SPACE_IN_BYTES: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x01a3057a_74d6_4e80_bea7_dc4c212ce50a), pid: 5 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STORAGE_FREE_SPACE_IN_OBJECTS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x01a3057a_74d6_4e80_bea7_dc4c212ce50a), pid: 6 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STORAGE_MAX_OBJECT_SIZE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x01a3057a_74d6_4e80_bea7_dc4c212ce50a), pid: 9 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_STORAGE_OBJECT_PROPERTIES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x01a3057a_74d6_4e80_bea7_dc4c212ce50a);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STORAGE_SERIAL_NUMBER: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x01a3057a_74d6_4e80_bea7_dc4c212ce50a), pid: 8 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_STORAGE_TYPE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x01a3057a_74d6_4e80_bea7_dc4c212ce50a), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_TASK_OBJECT_PROPERTIES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xe354e95e_d8a0_4637_a03a_0cb26838dbc7);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_TASK_OWNER: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xe354e95e_d8a0_4637_a03a_0cb26838dbc7), pid: 11 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_TASK_PERCENT_COMPLETE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xe354e95e_d8a0_4637_a03a_0cb26838dbc7), pid: 8 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_TASK_REMINDER_DATE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xe354e95e_d8a0_4637_a03a_0cb26838dbc7), pid: 10 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_TASK_STATUS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0xe354e95e_d8a0_4637_a03a_0cb26838dbc7), pid: 6 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_VIDEO_AUTHOR: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x346f2163_f998_4146_8b01_d19b4c00de9a), pid: 2 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_VIDEO_BITRATE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x346f2163_f998_4146_8b01_d19b4c00de9a), pid: 13 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_VIDEO_BUFFER_SIZE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x346f2163_f998_4146_8b01_d19b4c00de9a), pid: 8 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_VIDEO_CREDITS: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x346f2163_f998_4146_8b01_d19b4c00de9a), pid: 9 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_VIDEO_FOURCC_CODE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x346f2163_f998_4146_8b01_d19b4c00de9a), pid: 14 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_VIDEO_FRAMERATE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x346f2163_f998_4146_8b01_d19b4c00de9a), pid: 15 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_VIDEO_KEY_FRAME_DISTANCE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x346f2163_f998_4146_8b01_d19b4c00de9a), pid: 10 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_VIDEO_OBJECT_PROPERTIES_V1: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x346f2163_f998_4146_8b01_d19b4c00de9a);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_VIDEO_QUALITY_SETTING: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x346f2163_f998_4146_8b01_d19b4c00de9a), pid: 11 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_VIDEO_RECORDEDTV_CHANNEL_NUMBER: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x346f2163_f998_4146_8b01_d19b4c00de9a), pid: 5 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_VIDEO_RECORDEDTV_REPEAT: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x346f2163_f998_4146_8b01_d19b4c00de9a), pid: 7 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_VIDEO_RECORDEDTV_STATION_NAME: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x346f2163_f998_4146_8b01_d19b4c00de9a), pid: 4 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub const WPD_VIDEO_SCAN_TYPE: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY = super::super::UI::Shell::PropertiesSystem::PROPERTYKEY { fmtid: ::windows_sys::core::GUID::from_u128(0x346f2163_f998_4146_8b01_d19b4c00de9a), pid: 12 };
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WpdSerializer: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x0b91a74b_ad7c_4a9d_b563_29eef9167172);
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type DELETE_OBJECT_OPTIONS = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const PORTABLE_DEVICE_DELETE_NO_RECURSION: DELETE_OBJECT_OPTIONS = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const PORTABLE_DEVICE_DELETE_WITH_RECURSION: DELETE_OBJECT_OPTIONS = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type DEVICE_RADIO_STATE = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const DRS_RADIO_ON: DEVICE_RADIO_STATE = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const DRS_SW_RADIO_OFF: DEVICE_RADIO_STATE = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const DRS_HW_RADIO_OFF: DEVICE_RADIO_STATE = 2i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const DRS_SW_HW_RADIO_OFF: DEVICE_RADIO_STATE = 3i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const DRS_HW_RADIO_ON_UNCONTROLLABLE: DEVICE_RADIO_STATE = 4i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const DRS_RADIO_INVALID: DEVICE_RADIO_STATE = 5i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const DRS_HW_RADIO_OFF_UNCONTROLLABLE: DEVICE_RADIO_STATE = 6i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const DRS_RADIO_MAX: DEVICE_RADIO_STATE = 6i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type SMS_MESSAGE_TYPES = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const SMS_TEXT_MESSAGE: SMS_MESSAGE_TYPES = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const SMS_BINARY_MESSAGE: SMS_MESSAGE_TYPES = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type SYSTEM_RADIO_STATE = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const SRS_RADIO_ENABLED: SYSTEM_RADIO_STATE = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const SRS_RADIO_DISABLED: SYSTEM_RADIO_STATE = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type WPD_BITRATE_TYPES = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_BITRATE_TYPE_UNUSED: WPD_BITRATE_TYPES = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_BITRATE_TYPE_DISCRETE: WPD_BITRATE_TYPES = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_BITRATE_TYPE_VARIABLE: WPD_BITRATE_TYPES = 2i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_BITRATE_TYPE_FREE: WPD_BITRATE_TYPES = 3i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type WPD_CAPTURE_MODES = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CAPTURE_MODE_UNDEFINED: WPD_CAPTURE_MODES = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CAPTURE_MODE_NORMAL: WPD_CAPTURE_MODES = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CAPTURE_MODE_BURST: WPD_CAPTURE_MODES = 2i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CAPTURE_MODE_TIMELAPSE: WPD_CAPTURE_MODES = 3i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type WPD_COLOR_CORRECTED_STATUS_VALUES = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_COLOR_CORRECTED_STATUS_NOT_CORRECTED: WPD_COLOR_CORRECTED_STATUS_VALUES = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_COLOR_CORRECTED_STATUS_CORRECTED: WPD_COLOR_CORRECTED_STATUS_VALUES = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_COLOR_CORRECTED_STATUS_SHOULD_NOT_BE_CORRECTED: WPD_COLOR_CORRECTED_STATUS_VALUES = 2i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type WPD_COMMAND_ACCESS_TYPES = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_COMMAND_ACCESS_READ: WPD_COMMAND_ACCESS_TYPES = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_COMMAND_ACCESS_READWRITE: WPD_COMMAND_ACCESS_TYPES = 3i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_COMMAND_ACCESS_FROM_PROPERTY_WITH_STGM_ACCESS: WPD_COMMAND_ACCESS_TYPES = 4i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_COMMAND_ACCESS_FROM_PROPERTY_WITH_FILE_ACCESS: WPD_COMMAND_ACCESS_TYPES = 8i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_COMMAND_ACCESS_FROM_ATTRIBUTE_WITH_METHOD_ACCESS: WPD_COMMAND_ACCESS_TYPES = 16i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type WPD_CROPPED_STATUS_VALUES = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CROPPED_STATUS_NOT_CROPPED: WPD_CROPPED_STATUS_VALUES = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CROPPED_STATUS_CROPPED: WPD_CROPPED_STATUS_VALUES = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_CROPPED_STATUS_SHOULD_NOT_BE_CROPPED: WPD_CROPPED_STATUS_VALUES = 2i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type WPD_DEVICE_TRANSPORTS = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_DEVICE_TRANSPORT_UNSPECIFIED: WPD_DEVICE_TRANSPORTS = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_DEVICE_TRANSPORT_USB: WPD_DEVICE_TRANSPORTS = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_DEVICE_TRANSPORT_IP: WPD_DEVICE_TRANSPORTS = 2i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_DEVICE_TRANSPORT_BLUETOOTH: WPD_DEVICE_TRANSPORTS = 3i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type WPD_DEVICE_TYPES = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_DEVICE_TYPE_GENERIC: WPD_DEVICE_TYPES = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_DEVICE_TYPE_CAMERA: WPD_DEVICE_TYPES = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_DEVICE_TYPE_MEDIA_PLAYER: WPD_DEVICE_TYPES = 2i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_DEVICE_TYPE_PHONE: WPD_DEVICE_TYPES = 3i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_DEVICE_TYPE_VIDEO: WPD_DEVICE_TYPES = 4i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_DEVICE_TYPE_PERSONAL_INFORMATION_MANAGER: WPD_DEVICE_TYPES = 5i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_DEVICE_TYPE_AUDIO_RECORDER: WPD_DEVICE_TYPES = 6i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type WPD_EFFECT_MODES = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EFFECT_MODE_UNDEFINED: WPD_EFFECT_MODES = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EFFECT_MODE_COLOR: WPD_EFFECT_MODES = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EFFECT_MODE_BLACK_AND_WHITE: WPD_EFFECT_MODES = 2i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EFFECT_MODE_SEPIA: WPD_EFFECT_MODES = 3i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type WPD_EXPOSURE_METERING_MODES = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EXPOSURE_METERING_MODE_UNDEFINED: WPD_EXPOSURE_METERING_MODES = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EXPOSURE_METERING_MODE_AVERAGE: WPD_EXPOSURE_METERING_MODES = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EXPOSURE_METERING_MODE_CENTER_WEIGHTED_AVERAGE: WPD_EXPOSURE_METERING_MODES = 2i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EXPOSURE_METERING_MODE_MULTI_SPOT: WPD_EXPOSURE_METERING_MODES = 3i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EXPOSURE_METERING_MODE_CENTER_SPOT: WPD_EXPOSURE_METERING_MODES = 4i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type WPD_EXPOSURE_PROGRAM_MODES = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EXPOSURE_PROGRAM_MODE_UNDEFINED: WPD_EXPOSURE_PROGRAM_MODES = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EXPOSURE_PROGRAM_MODE_MANUAL: WPD_EXPOSURE_PROGRAM_MODES = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EXPOSURE_PROGRAM_MODE_AUTO: WPD_EXPOSURE_PROGRAM_MODES = 2i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EXPOSURE_PROGRAM_MODE_APERTURE_PRIORITY: WPD_EXPOSURE_PROGRAM_MODES = 3i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EXPOSURE_PROGRAM_MODE_SHUTTER_PRIORITY: WPD_EXPOSURE_PROGRAM_MODES = 4i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EXPOSURE_PROGRAM_MODE_CREATIVE: WPD_EXPOSURE_PROGRAM_MODES = 5i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EXPOSURE_PROGRAM_MODE_ACTION: WPD_EXPOSURE_PROGRAM_MODES = 6i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_EXPOSURE_PROGRAM_MODE_PORTRAIT: WPD_EXPOSURE_PROGRAM_MODES = 7i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type WPD_FLASH_MODES = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_FLASH_MODE_UNDEFINED: WPD_FLASH_MODES = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_FLASH_MODE_AUTO: WPD_FLASH_MODES = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_FLASH_MODE_OFF: WPD_FLASH_MODES = 2i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_FLASH_MODE_FILL: WPD_FLASH_MODES = 3i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_FLASH_MODE_RED_EYE_AUTO: WPD_FLASH_MODES = 4i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_FLASH_MODE_RED_EYE_FILL: WPD_FLASH_MODES = 5i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_FLASH_MODE_EXTERNAL_SYNC: WPD_FLASH_MODES = 6i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type WPD_FOCUS_METERING_MODES = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_FOCUS_METERING_MODE_UNDEFINED: WPD_FOCUS_METERING_MODES = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_FOCUS_METERING_MODE_CENTER_SPOT: WPD_FOCUS_METERING_MODES = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_FOCUS_METERING_MODE_MULTI_SPOT: WPD_FOCUS_METERING_MODES = 2i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type WPD_FOCUS_MODES = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_FOCUS_UNDEFINED: WPD_FOCUS_MODES = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_FOCUS_MANUAL: WPD_FOCUS_MODES = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_FOCUS_AUTOMATIC: WPD_FOCUS_MODES = 2i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_FOCUS_AUTOMATIC_MACRO: WPD_FOCUS_MODES = 3i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type WPD_META_GENRES = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_META_GENRE_UNUSED: WPD_META_GENRES = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_META_GENRE_GENERIC_MUSIC_AUDIO_FILE: WPD_META_GENRES = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_META_GENRE_GENERIC_NON_MUSIC_AUDIO_FILE: WPD_META_GENRES = 17i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_META_GENRE_SPOKEN_WORD_AUDIO_BOOK_FILES: WPD_META_GENRES = 18i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_META_GENRE_SPOKEN_WORD_FILES_NON_AUDIO_BOOK: WPD_META_GENRES = 19i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_META_GENRE_SPOKEN_WORD_NEWS: WPD_META_GENRES = 20i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_META_GENRE_SPOKEN_WORD_TALK_SHOWS: WPD_META_GENRES = 21i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_META_GENRE_GENERIC_VIDEO_FILE: WPD_META_GENRES = 33i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_META_GENRE_NEWS_VIDEO_FILE: WPD_META_GENRES = 34i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_META_GENRE_MUSIC_VIDEO_FILE: WPD_META_GENRES = 35i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_META_GENRE_HOME_VIDEO_FILE: WPD_META_GENRES = 36i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_META_GENRE_FEATURE_FILM_VIDEO_FILE: WPD_META_GENRES = 37i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_META_GENRE_TELEVISION_VIDEO_FILE: WPD_META_GENRES = 38i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_META_GENRE_TRAINING_EDUCATIONAL_VIDEO_FILE: WPD_META_GENRES = 39i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_META_GENRE_PHOTO_MONTAGE_VIDEO_FILE: WPD_META_GENRES = 40i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_META_GENRE_GENERIC_NON_AUDIO_NON_VIDEO: WPD_META_GENRES = 48i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_META_GENRE_AUDIO_PODCAST: WPD_META_GENRES = 64i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_META_GENRE_VIDEO_PODCAST: WPD_META_GENRES = 65i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_META_GENRE_MIXED_PODCAST: WPD_META_GENRES = 66i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type WPD_OPERATION_STATES = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OPERATION_STATE_UNSPECIFIED: WPD_OPERATION_STATES = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OPERATION_STATE_STARTED: WPD_OPERATION_STATES = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OPERATION_STATE_RUNNING: WPD_OPERATION_STATES = 2i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OPERATION_STATE_PAUSED: WPD_OPERATION_STATES = 3i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OPERATION_STATE_CANCELLED: WPD_OPERATION_STATES = 4i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OPERATION_STATE_FINISHED: WPD_OPERATION_STATES = 5i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_OPERATION_STATE_ABORTED: WPD_OPERATION_STATES = 6i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type WPD_PARAMETER_USAGE_TYPES = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_PARAMETER_USAGE_RETURN: WPD_PARAMETER_USAGE_TYPES = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_PARAMETER_USAGE_IN: WPD_PARAMETER_USAGE_TYPES = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_PARAMETER_USAGE_OUT: WPD_PARAMETER_USAGE_TYPES = 2i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_PARAMETER_USAGE_INOUT: WPD_PARAMETER_USAGE_TYPES = 3i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type WPD_POWER_SOURCES = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_POWER_SOURCE_BATTERY: WPD_POWER_SOURCES = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_POWER_SOURCE_EXTERNAL: WPD_POWER_SOURCES = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type WPD_RENDERING_INFORMATION_PROFILE_ENTRY_TYPES = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_RENDERING_INFORMATION_PROFILE_ENTRY_TYPE_OBJECT: WPD_RENDERING_INFORMATION_PROFILE_ENTRY_TYPES = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_RENDERING_INFORMATION_PROFILE_ENTRY_TYPE_RESOURCE: WPD_RENDERING_INFORMATION_PROFILE_ENTRY_TYPES = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type WPD_SECTION_DATA_UNITS_VALUES = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_SECTION_DATA_UNITS_BYTES: WPD_SECTION_DATA_UNITS_VALUES = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_SECTION_DATA_UNITS_MILLISECONDS: WPD_SECTION_DATA_UNITS_VALUES = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type WPD_SERVICE_INHERITANCE_TYPES = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_SERVICE_INHERITANCE_IMPLEMENTATION: WPD_SERVICE_INHERITANCE_TYPES = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type WPD_SMS_ENCODING_TYPES = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const SMS_ENCODING_7_BIT: WPD_SMS_ENCODING_TYPES = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const SMS_ENCODING_8_BIT: WPD_SMS_ENCODING_TYPES = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const SMS_ENCODING_UTF_16: WPD_SMS_ENCODING_TYPES = 2i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type WPD_STORAGE_ACCESS_CAPABILITY_VALUES = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_STORAGE_ACCESS_CAPABILITY_READWRITE: WPD_STORAGE_ACCESS_CAPABILITY_VALUES = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_STORAGE_ACCESS_CAPABILITY_READ_ONLY_WITHOUT_OBJECT_DELETION: WPD_STORAGE_ACCESS_CAPABILITY_VALUES = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_STORAGE_ACCESS_CAPABILITY_READ_ONLY_WITH_OBJECT_DELETION: WPD_STORAGE_ACCESS_CAPABILITY_VALUES = 2i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type WPD_STORAGE_TYPE_VALUES = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_STORAGE_TYPE_UNDEFINED: WPD_STORAGE_TYPE_VALUES = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_STORAGE_TYPE_FIXED_ROM: WPD_STORAGE_TYPE_VALUES = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_STORAGE_TYPE_REMOVABLE_ROM: WPD_STORAGE_TYPE_VALUES = 2i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_STORAGE_TYPE_FIXED_RAM: WPD_STORAGE_TYPE_VALUES = 3i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_STORAGE_TYPE_REMOVABLE_RAM: WPD_STORAGE_TYPE_VALUES = 4i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type WPD_STREAM_UNITS = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_STREAM_UNITS_BYTES: WPD_STREAM_UNITS = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_STREAM_UNITS_FRAMES: WPD_STREAM_UNITS = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_STREAM_UNITS_ROWS: WPD_STREAM_UNITS = 2i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_STREAM_UNITS_MILLISECONDS: WPD_STREAM_UNITS = 4i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_STREAM_UNITS_MICROSECONDS: WPD_STREAM_UNITS = 8i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type WPD_VIDEO_SCAN_TYPES = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_VIDEO_SCAN_TYPE_UNUSED: WPD_VIDEO_SCAN_TYPES = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_VIDEO_SCAN_TYPE_PROGRESSIVE: WPD_VIDEO_SCAN_TYPES = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_VIDEO_SCAN_TYPE_FIELD_INTERLEAVED_UPPER_FIRST: WPD_VIDEO_SCAN_TYPES = 2i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_VIDEO_SCAN_TYPE_FIELD_INTERLEAVED_LOWER_FIRST: WPD_VIDEO_SCAN_TYPES = 3i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_VIDEO_SCAN_TYPE_FIELD_SINGLE_UPPER_FIRST: WPD_VIDEO_SCAN_TYPES = 4i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_VIDEO_SCAN_TYPE_FIELD_SINGLE_LOWER_FIRST: WPD_VIDEO_SCAN_TYPES = 5i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_VIDEO_SCAN_TYPE_MIXED_INTERLACE: WPD_VIDEO_SCAN_TYPES = 6i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_VIDEO_SCAN_TYPE_MIXED_INTERLACE_AND_PROGRESSIVE: WPD_VIDEO_SCAN_TYPES = 7i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type WPD_WHITE_BALANCE_SETTINGS = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_WHITE_BALANCE_UNDEFINED: WPD_WHITE_BALANCE_SETTINGS = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_WHITE_BALANCE_MANUAL: WPD_WHITE_BALANCE_SETTINGS = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_WHITE_BALANCE_AUTOMATIC: WPD_WHITE_BALANCE_SETTINGS = 2i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_WHITE_BALANCE_ONE_PUSH_AUTOMATIC: WPD_WHITE_BALANCE_SETTINGS = 3i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_WHITE_BALANCE_DAYLIGHT: WPD_WHITE_BALANCE_SETTINGS = 4i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_WHITE_BALANCE_FLORESCENT: WPD_WHITE_BALANCE_SETTINGS = 5i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_WHITE_BALANCE_TUNGSTEN: WPD_WHITE_BALANCE_SETTINGS = 6i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_WHITE_BALANCE_FLASH: WPD_WHITE_BALANCE_SETTINGS = 7i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type WpdAttributeForm = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_PROPERTY_ATTRIBUTE_FORM_UNSPECIFIED: WpdAttributeForm = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_PROPERTY_ATTRIBUTE_FORM_RANGE: WpdAttributeForm = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_PROPERTY_ATTRIBUTE_FORM_ENUMERATION: WpdAttributeForm = 2i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_PROPERTY_ATTRIBUTE_FORM_REGULAR_EXPRESSION: WpdAttributeForm = 3i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_PROPERTY_ATTRIBUTE_FORM_OBJECT_IDENTIFIER: WpdAttributeForm = 4i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub type WpdParameterAttributeForm = i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_PARAMETER_ATTRIBUTE_FORM_UNSPECIFIED: WpdParameterAttributeForm = 0i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_PARAMETER_ATTRIBUTE_FORM_RANGE: WpdParameterAttributeForm = 1i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_PARAMETER_ATTRIBUTE_FORM_ENUMERATION: WpdParameterAttributeForm = 2i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_PARAMETER_ATTRIBUTE_FORM_REGULAR_EXPRESSION: WpdParameterAttributeForm = 3i32;
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`*"]
pub const WPD_PARAMETER_ATTRIBUTE_FORM_OBJECT_IDENTIFIER: WpdParameterAttributeForm = 4i32;
#[repr(C)]
#[doc = "*Required features: `\"Win32_Devices_PortableDevices\"`, `\"Win32_UI_Shell_PropertiesSystem\"`*"]
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
pub struct WPD_COMMAND_ACCESS_LOOKUP_ENTRY {
    pub Command: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY,
    pub AccessType: u32,
    pub AccessProperty: super::super::UI::Shell::PropertiesSystem::PROPERTYKEY,
}
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
impl ::core::marker::Copy for WPD_COMMAND_ACCESS_LOOKUP_ENTRY {}
#[cfg(feature = "Win32_UI_Shell_PropertiesSystem")]
impl ::core::clone::Clone for WPD_COMMAND_ACCESS_LOOKUP_ENTRY {
    fn clone(&self) -> Self {
        *self
    }
}
